
package uoc.ei.exemples.modul1.generics;

/** Implementaci del TAD Conjunt que fa servir una taula com a representaci per
 * a guardar els elements. Aquesta implementaci t la restricci de que hi ha que
 * especificar en el moment de la creaci del conjunt el nombre d'elements mxim que
 * es guardaran en el conjunt.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */

public class ConjuntVectorImpl<E> implements ConjuntAfitat<E> {

	/** Taula que guarda els elements del conjunt. */

	private E[] elements;


	/** Nmero d'elements que el conjunt cont actualment. */

	private int numElementsActual;


	/** Crea un conjunt amb un nombre mxim d'elements.
	 * @pre n>=0
	 * @post elements.length==n && numElementsActual==0
	 * @param n Nombre mxim d'elements del conjunt.
	 */

	public ConjuntVectorImpl(int n) {
		elements=(E[])new Object[n];
		numElementsActual=0;
	}


	/**
	 * @pre elem!=null && (numElementsActual<elements.length || hiEs(elem))
	 * @post this.hiEs(elem) &&
	 * 		($old(this).hiEs(elem) && $old(numElementsActual)==numElementsActual ||
	 * 		 !$old(this).hiEs(elem) && $old(numElementsActual)+1==numElementsActual) &&
     * 		$all(i:elements,i>=$old(numElementsActual) || $old(elements[i])==elements[i])
	 */

	public void afegir(E elem) {
		if (!hiEs(elem)) {
			elements[numElementsActual]=elem;
			numElementsActual++;
		}
	}


	/**
	 * @pre elem!=null
	 * @post $old(this)==this &&
	 * 		 $return == $exists(i:elements,i<numElementsActual && elements[i].equals(elem))
	 */

	public boolean hiEs(E elem) {
		return cercarPosicioElement(elem)!=-1;
	}


	/**
	 * @pre elem!=null
	 * @post !hiEs(elem) &&
	 * 		($old(this).hiEs(elem) && $old(numElementsActual)==numElementsActual+1 ||
	 * 		 !$old(this).hiEs(elem) && $old(numElementsActual)==numElementsActual) &&
     * 		$all(i:elements,i>=$old(numElementsActual) ||
     * 						$old(elements[i])==elem ||
     * 						this.hiEs($old(elements[i])))
	 */

	public E esborrar(E elem) {
		E elementEsborrat=null;
		int posicio=cercarPosicioElement(elem);
		if (posicio!=-1) {
			elementEsborrat=elements[posicio];
			numElementsActual--;
			if (posicio<numElementsActual)
				elements[posicio]=elements[numElementsActual];
		}
		return elementEsborrat;
	}


	/**
	 * @post $return == (numElementsActual==0)
	 */

	public boolean estaBuit() {
		return numElementsActual==0;
	}


	/**
	 * @post $return == (numElementsActual==elements.length)
	 */

	public boolean estaPle() {
		return numElementsActual==elements.length;
	}


	/** Mtode privat que cerca la posici d'un element
	 * dins de la taula on es guarden tots els elements
	 * que s'han afegit al conjunt.
	 * @pre elem!=null
	 * @post $old(this)==this &&
	 * 		 ($return == -1 && !hiEs(elem) ||
	 * 		  elements[$return].equals(elem))
	 * @param elem
	 * @return
	 */

	protected int cercarPosicioElement(E elem) {
		boolean trobat=false;
		int i=0;
		while (i<numElementsActual && !trobat) {
			trobat=elements[i].equals(elem);
			if (!trobat) i++;
		}
		return trobat ? i : -1;
	}


	/** Definici d'aquest mtode per a poder obtenir una representaci
	 * String del conjunt on es vegin tots els elements.
	 */

	public String toString() {
		StringBuffer sb=new StringBuffer();
		sb.append('{');
		int i=0;
		while (i<numElementsActual) {
			sb.append(elements[i]);
			i++;
			if (i<numElementsActual) sb.append(',');
		}
		sb.append('}');
		return sb.toString();
	}

}
