package uoc.ei.exemples.modul3.cua;

import uoc.ei.tads.Cua;

/** Simula un processador d'operacions que pot realitzar una operaci
 * cada TEMPS_PROCESSAMENT_OPERACIO segons.
 * Aquest processador llegeix les operacions a realitzar d'una cua. Cada cop
 * que realitza una operaci n'escriu el resultat per pantalla.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */

public class Processador extends Thread {

	private static final int TEMPS_PROCESSAMENT_OPERACIO = 10;

	private volatile Cua<Operacio> operacions;

	public Processador(Cua<Operacio> cua) {
		super();
		operacions=cua;
	}

	public void run() {
		// El try/catch s necessari per la instrucci sleep
		// (veure documentaci del propi JDK)
		try {
			// cos principal del processador: va llegint operacions
			// de la cua, i les va processant, amb la espera corresponent
			// (que simula un temps de processament fora alt)
			while (true) {
				// Aquest bloc de sincronitzaci s necessari per a evitar que el thread
				// del processador i el d'usuari (aquest main) intentin accedir a la vegada
				// a la cua.
				synchronized (operacions) {
					// noms processarem la segent operaci de la cua, si
					// aquesta est disponible
					if (!operacions.estaBuit()) {
						Operacio op=operacions.desencuar();
						System.out.println("processant: "+op);
						System.out.println("resultat: "+op.calcular());
						System.out.println("nombre d'operacions pendents: "+operacions.nombreElems());
					}
				}
				sleep(TEMPS_PROCESSAMENT_OPERACIO*1000);
			}
		} catch (InterruptedException e) {
			System.out.println("Thread interromput mentre estava esperant processar la segent operaci.");
		}
	}

}
