
package uoc.ei.exemples.modul3.llista;

import java.io.IOException;

import uoc.ei.exemples.modul1.generics.Conjunt;
import uoc.ei.tads.*;

/** Implementaci del TAD Conjunt que fa servir una taula com a representaci per
 * a guardar els elements. Aquesta implementaci t la restricci de que hi ha que
 * especificar en el moment de la creaci del conjunt el nombre d'elements mxim que
 * es guardaran en el conjunt.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */

public class ConjuntLlistaImpl<E> implements Conjunt<E> {

	/** Llista que usem per a representar els elements
	 * del Conjunt. */

	private Llista<E> llistaDeElements;


	public ConjuntLlistaImpl() {
		llistaDeElements=new LlistaEncadenada<E>();
	}


	public void afegir(E elem) {
		if (!hiEs(elem))
			llistaDeElements.afegirAlFinal(elem);
	}


	public boolean hiEs(E elem) {
		boolean trobat=false;
		Iterador<E> iter=llistaDeElements.elements();
		while (!trobat && iter.hiHaSeguent())
			trobat=elem.equals(iter.seguent());
		return trobat;
	}


	public E esborrar(E elem) {
		E elementEsborrat=null;
		boolean trobat=false;
		Recorregut<E> rec=llistaDeElements.posicions();
		Posicio<E> anterior=null,actual=null;
		while (!trobat && rec.hiHaSeguent()) {
			anterior=actual;
			actual=rec.seguent();
			trobat=actual!=null && elem.equals(actual.getElem());
		}
		if (trobat)
			elementEsborrat=llistaDeElements.esborrarSeguent(anterior);
		return elementEsborrat;
	}


	public boolean estaBuit() {
		return llistaDeElements.estaBuit();
	}


	public String toString() {
		return Utilitats.toStringContenidor("ConjuntLlistaImpl",llistaDeElements.posicions());
	}


	public static void main(String[] args) {
		System.out.println("Introdueix nombres naturals a afegir al conjunt");
		boolean sortir=false;
		Conjunt<Integer> cjt=new ConjuntLlistaImpl<Integer>();
		while (!sortir) {
			try {
				String elem=Utilitats.llegirString("Introdueix un nombre (o \"fi\" per acabar): ",System.in);
				sortir=elem.equalsIgnoreCase("fi");
				if (!sortir)
					cjt.afegir(Integer.parseInt(elem));
			} catch (NumberFormatException e) {
				System.out.println("El text introdut no s un nombre, torna-ho a provar.");
			} catch (IOException e) {
				System.out.println("Problema amb l'entrada, avortant el programa");
				sortir=true;
			}
		}
		System.out.println("El conjunt resultant s: "+cjt);
	}

}
