package uoc.ei.exemples.modul3.redimensionament;

import uoc.ei.tads.*;

/** Variaci de la implementaci d'una Cua que redimensiona el vector on
 * es guarden els elements de manera automtica quan aquest est ple.
 * La implementaci s prcticament igual a Cua excepte per aix i el
 * fet que com a conseqncia no s'implementa la interfcie ContenidorAfitat.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class CuaRedimensionableImpl<E>  implements Cua<E> {

   private static final long serialVersionUID = 1;
	
	
   //-------------------------------------------------------
   // La segent part de la implementaci s idntica
   // a la de CuaVectorImpl.
   // No la podem reaprofitar donat que no volem que
   // CuaRedimensionableImpl implementi ContenidorAfitat.
   // (per a poder reaprofitar-la caldria modificar la
   // llibreria de tads de l'assignatura i crear una classe
   // pare com a ambdues implementacions que no implements
   // ContenidorAfitat)
   //-------------------------------------------------------

   /** Capacitat mxima, per defecte, del contenidor. */
   public static final int MIDA_INICIAL = 10;

   /** Taula d'elements del contenidor. */
   protected E[] elements;

   /** Nombre d'elements que hi ha actualment al contenidor. */
   protected int n;

   /** Primer element de la cua. */
   private int  primer;

   public CuaRedimensionableImpl() {
      elements = (E[])new Object[MIDA_INICIAL];
      n = 0;
      primer = 0;
   }

   private int posicio(int posicio) { return posicio % elements.length; }
   private int seguent(int posicio) { return (posicio+1)==elements.length ? 0 : posicio+1; }
   private boolean estaPle() { return  ( n == elements.length ); }

   public int nombreElems() { return  n; }
   public boolean estaBuit() { return  ( n == 0 ); }
   public E primer() { return  elements[primer]; }
   
   
   public E desencuar() {
      E elem = elements[primer];
      elements[primer]=null;
      primer = seguent(primer);
      n--;
      return  elem;
   }

   public Iterador<E> elements() {
		return new IteradorVectorImpl<E>(elements,nombreElems(),primer);
   }

   /**
    * Mtode que sobreescriu Object.toString(). Treu els elements separats
    * pel salt de lnia de la plataforma.
    * @return  llistat dels elements
    */
   public String toString()
   {
    StringBuffer buffer = new StringBuffer();
    buffer.append("{CUA:");
    for (Iterador<E> it = elements(); it.hiHaSeguent(); ) {
       buffer.append(it.seguent());
       if (it.hiHaSeguent()) buffer.append(',');
    }
    buffer.append("}");
    return  buffer.toString();
   }

   
   //-------------------------------------------------------
   // modificacions sobre la implementaci: CuaVectorImpl
   //-------------------------------------------------------

   /** La implementaci d'encuar comprova si el vector est ple, i
    * en aquest cas redimensiona el vector.
    */

   public void encuar(E elem) {
	   if (estaPle())
		   redimensionar();
	   int darrer = posicio(primer + n);
	   elements[darrer] = elem;
   	   n++;
   }
   
   
   /** Aquest mtode duplica la mida del vector on guardem
    * els elements.
    */
   
   private void redimensionar() {
	   // creaci del nou vector (amb el doble de capacitat)
	   E[] auxElements = (E[])new Object[elements.length*2];
	   // cpia dels elements d'un a l'altre
	   Iterador<E> it=elements();
	   int i=0;
	   while (it.hiHaSeguent()) {
		   auxElements[i]=it.seguent();
		   i++;
	   }
	   // substituci del vector antic ple pel nou amb ms capacitat
	   primer=0;
	   elements=auxElements;
   }

}
