package uoc.ei.exemples.modul3.referencia;

import uoc.ei.tads.*;

/** Implementaci encadenada de la col.lecci Cua
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class CuaEncadenadaImpl<E>  implements Cua<E> {

	private static final long serialVersionUID = 1;

   /** Nombre d'elements que hi ha actualment al contenidor. */
   protected int n;

   /** Primer element de la cua. */
   private Node<E> primer;

   /** Darrer element de la cua. */
   private Node<E> darrer;

   public CuaEncadenadaImpl() {
	   primer=null;
	   darrer=null;
   }


   /**
    * Accessor de lectura del nombre d'elements que hi ha al contenidor.
    * @return  nombre d'elements que cont actualment
    */
   public int nombreElems() { return  n; }


   /**
    * Mtode per a comprovar si el contenidor est buit.
    * @return  cert o fals, segons si est buit o no ho est
    */
   public boolean estaBuit() { return  ( n == 0 ); }


   /**
    * Afegeix un element a la cua, si hi cap.
    * @exception ExcepcioContenidorPle  si la cua est plena
    * @param elem  element que es vol afegir a la cua
    */
   public void encuar(E elem) {
	   Node<E> nou=new Node<E>(elem);
	   if (primer==null)
		   primer=nou;
	   else
		   darrer.setSeguent(nou);
	   darrer=nou;
	   n++;
   }


   /**
    * Esborra el primer element de la cua, si n'hi ha algun.
    * @exception ExcepcioContenidorBuit  si la cua est buida
    * @return  primer element inserit a la cua
    * @pre primer!=null
    */
   public E desencuar() {
	   Node<E> aux=primer;
	   primer=primer.getSeguent();
	   if (primer==null)
		   darrer=null;
	   n--;
	   return aux.getElem();
   }

   /**
    * Accessor de lectura del primer element afegit a la cua, si n'hi ha.
    * @exception ExcepcioContenidorBuit  si la cua est buida
    * @return  primer element de la cua
    * @pre primer!=null
    */
   public E primer() {
	   return primer.getElem();
   }

   /**
    * Accessor de lectura dels elements que hi ha al contenidor.
    * @return  enumeraci dels elements del contenidor
    */
   public Iterador<E> elements() {
	   return new IteradorCua<E>(this);
   }

   /**
    * Classe que implementa un node amb un encadenament a node, que uneix un
    * node amb el segent en una estructura de dades amb encadenament simple.
    */
   protected static class Node<EN>  implements Posicio<EN> {

      /** Atribut que en Java 1.5 determina la compatibilitat entre objectes
       * serialitzables de la mateixa classe. L'deitificador es calcula
       * mitjanant un mtode de la classe Utilitats. */
      private static final long serialVersionUID = 1;


      /** Element contingut al node. */
      private EN element;

      /** Encadenament al node segent. */
      private Node<EN> seg;

      public Node(EN elem) {
         element = elem;
         seg = null;
      }

      public EN getElem() { return  element; }
      public Node<EN> getSeguent() { return  seg; }
      public void setSeguent(Node<EN> node) { seg=node; }

   }


   /** Classe que proporciona un recorregut de les posicions de la cua.
    */
   protected static class IteradorCua<ER>  implements Iterador<ER> {

    private static final long serialVersionUID = 1;

	/** Segent node a visitar. */
    protected Node<ER> nodeActual;

    public IteradorCua(CuaEncadenadaImpl<ER> cua) {
      	nodeActual=cua.primer;
      }

	/** Comprova si queden posicions per visitar.
	 * @return  cert si queden posicions i fals en cas contrari.
	 */
	public boolean hiHaSeguent() { return  nodeActual!=null; }

	/** Retorna el seguent node a visitar i avana.
	 * @exception ExcepcioPosicioInvalida  si la segent posici no
	 *            existeix
	 * @return  segent posici.
	 * @pre hiHaSeguent(), new ExcepcioPosicioInvalida("no hi ha seguent")
	 */
	public ER seguent() {
		Node<ER> aux=nodeActual;
		nodeActual = nodeActual.getSeguent();
		return  aux.getElem();
	}
   }

}
