package uoc.ei.exemples.modul4.fitxers;

import java.io.IOException;
import java.util.StringTokenizer;

import uoc.ei.tads.Iterador;
import uoc.ei.tads.Utilitats;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class ProvaSF {

	protected static void mostrarAjuda() {
		System.out.println("Simulaci d'una lnia de comandes amb comandes bsiques per a un sistema de fitxers");
		System.out.println("Comandes acceptades:");
		System.out.println("?, ajuda : mostra aquesta pantalla.");
		System.out.println("cd .. : puja al directori pare.");
		System.out.println("cd <dir> : va al directori fill anomenat <dir>.");
		System.out.println("md <dir> : crea un nou directori fill anomenat <dir>.");
		System.out.println("mf <fitxer> <contingut> : crea un nou fitxer de text amb el contingut especificat (el contingut no pot contenir espais).");
		System.out.println("ls : llista el contingut del directori actual.");
		System.out.println("cat : mostra el contingut d'un fitxer.");
		System.out.println("rm : elimina un directori o fitxer.");
		System.out.println("mv : canvia el nom d'un directori o fitxer.");
		System.out.println("exit, quit, stop, sortir : acaba el simulador");
		System.out.println();
		System.out.println("S'ha de tenir en compte que no hi ha cap control d'errors en la lectura de les comandes!");
	}


	protected static boolean executarComanda(SistemaDeFitxers sistemaDeFitxers,String comanda) throws ExcepcioSistemaDeFitxers {
		boolean sortir=false;
		StringTokenizer elementsComanda=new StringTokenizer(comanda);
		if (elementsComanda.countTokens()>0) {
			String tipusComanda=elementsComanda.nextToken();
			if (tipusComanda.equalsIgnoreCase("ajuda") ||
					tipusComanda.equals("?"))
				mostrarAjuda();
			else if (tipusComanda.equalsIgnoreCase("cd")) {
				String nouDir=elementsComanda.nextToken();
				if (nouDir.equals(".."))
					sistemaDeFitxers.pujaDirectori();
				else
					sistemaDeFitxers.baixaDirectori(nouDir);
			}
			else if (tipusComanda.equalsIgnoreCase("md")) {
				String nouDir=elementsComanda.nextToken();
				sistemaDeFitxers.crearDirectori(nouDir);
			}
			else if (tipusComanda.equalsIgnoreCase("mf")) {
				String nouFitxer=elementsComanda.nextToken();
				String contingut=elementsComanda.nextToken();
				sistemaDeFitxers.crearFitxer(nouFitxer,contingut);
			}
			else if (tipusComanda.equalsIgnoreCase("ls")) {
				Iterador<EntradaSF> entrades=sistemaDeFitxers.elements();
				while (entrades.hiHaSeguent()) {
					EntradaSF entrada=entrades.seguent();
					System.out.print(entrada);
					if (entrada.esDirectori())
						System.out.println("       <directori>");
					else
						System.out.println();
				}
			}
			else if (tipusComanda.equalsIgnoreCase("cat")) {
				String fitxer=elementsComanda.nextToken();
				System.out.println(sistemaDeFitxers.getContingutFitxer(fitxer));
			}
			else if (tipusComanda.equalsIgnoreCase("rm")) {
				String entrada=elementsComanda.nextToken();
				sistemaDeFitxers.eliminarEntrada(entrada);
			}
			else if (tipusComanda.equalsIgnoreCase("mv")) {
				String nomVell=elementsComanda.nextToken();
				String nomNou=elementsComanda.nextToken();
				sistemaDeFitxers.reanomenar(nomVell,nomNou);
			}
			else if ("exit,quit,sortir,stop".indexOf(tipusComanda)!=-1)
				sortir=true;
			else {
				System.out.println("Comanda no reconeguda. Pulsa ? per veure ajuda.");
			}
		}
		return sortir;
	}


	public static void main(String[] args) {
		SistemaDeFitxers sistemaDeFitxers=new SistemaDeFitxers("arrel");
		boolean sortir=false;
		try {
			while (!sortir) {
				try {
					String comanda=Utilitats.llegirString(sistemaDeFitxers.getDirectoriDeTreball()+"> ",System.in);
					comanda=comanda.trim();
					sortir=executarComanda(sistemaDeFitxers,comanda);
				} catch (ExcepcioSistemaDeFitxers e) {
					System.out.println(e.getMessage());
				}
			}
		} catch (IOException e) {
				System.out.println("Problema al llegir de l'entrada: "+e);
				System.out.println("Abortant el programa");
		}
	}

}
