package uoc.ei.exemples.modul4.fitxers;

import uoc.ei.tads.*;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class SistemaDeFitxers {

	private Arbre<EntradaSF> arbreDeDirectoris;
	private Posicio<EntradaSF> directoriDeTreball;

	public SistemaDeFitxers(String nomDirectoriArrel) {
		arbreDeDirectoris=new ArbreGeneralDelegImpl<EntradaSF>();
		Directori directoriArrel=new Directori(nomDirectoriArrel,null);
		directoriDeTreball=arbreDeDirectoris.afegir(null,directoriArrel);
	}


	public String getDirectoriDeTreball() {
		return directoriDeTreball.toString();
	}


	public void crearDirectori(String nom) throws ExcepcioSistemaDeFitxers {
		if (cercaFill(nom)!=null)
			throw new ExcepcioSistemaDeFitxers(directoriDeTreball+" ja t un element amb nom "+nom);
		Directori subdirectori=new Directori(nom,directoriDeTreball);
		arbreDeDirectoris.afegir(directoriDeTreball,subdirectori);
	}


	public void crearFitxer(String nom,String contingut) throws ExcepcioSistemaDeFitxers {
		if (cercaFill(nom)!=null)
			throw new ExcepcioSistemaDeFitxers(directoriDeTreball+" ja t un element amb nom "+nom);
		FitxerDeText fitxer=new FitxerDeText(nom,directoriDeTreball);
		arbreDeDirectoris.afegir(directoriDeTreball,fitxer);
		fitxer.setContingut(contingut);
	}


	public void eliminarEntrada(String nom) throws ExcepcioSistemaDeFitxers {
		Posicio<EntradaSF> entradaAEsborrar=cercaFill(nom);
		if (entradaAEsborrar==null)
			throw new ExcepcioSistemaDeFitxers(directoriDeTreball+" no t cap element amb nom "+nom);
		arbreDeDirectoris.esborrar(directoriDeTreball,entradaAEsborrar);
	}


	public Iterador<EntradaSF> elements() {
		Recorregut<EntradaSF> fills=arbreDeDirectoris.fills(directoriDeTreball);
		return new IteradorRecorregutImpl<EntradaSF,EntradaSF>(fills);
	}


	public void reanomenar(String nomVell, String nomNou) throws ExcepcioSistemaDeFitxers {
		Posicio<EntradaSF> entradaAReanomenar=cercaFill(nomVell);
		if (entradaAReanomenar==null)
			throw new ExcepcioSistemaDeFitxers(directoriDeTreball+" no t cap element amb nom "+nomVell);
		entradaAReanomenar.getElem().setNom(nomNou);
	}


	public void baixaDirectori(String nom) throws ExcepcioSistemaDeFitxers {
		Posicio<EntradaSF> fill=cercaFill(nom);
		if (fill==null)
			throw new ExcepcioSistemaDeFitxers(directoriDeTreball+" no t cap element amb nom "+nom);
		if (!fill.getElem().esDirectori())
			throw new ExcepcioSistemaDeFitxers(fill.getElem().getNom()+" no s un directori");
		directoriDeTreball=fill;
	}


	public void pujaDirectori() throws ExcepcioSistemaDeFitxers {
		Posicio<EntradaSF> pare=directoriDeTreball.getElem().getPare();
		if (pare==null)
			throw new ExcepcioSistemaDeFitxers(directoriDeTreball+" s el directori arrel");
		directoriDeTreball=pare;
	}


	public String getContingutFitxer(String nom) throws ExcepcioSistemaDeFitxers {
		Posicio<EntradaSF> fill=cercaFill(nom);
		if (fill==null)
			throw new ExcepcioSistemaDeFitxers(directoriDeTreball+" no t cap element amb nom "+nom);
		if (fill.getElem().esDirectori())
			throw new ExcepcioSistemaDeFitxers(fill.getElem().getNom()+" no s un fitxer");
		return ((FitxerDeText)fill.getElem()).getContingut();
	}


	private Posicio<EntradaSF> cercaFill(String nom) {
		Recorregut<EntradaSF> fills=arbreDeDirectoris.fills(directoriDeTreball);
		Posicio<EntradaSF> fillCercat=null;
		while (fills.hiHaSeguent() && fillCercat==null) {
			Posicio<EntradaSF> fill=fills.seguent();
			if (fill.getElem().getNom().equals(nom))
				fillCercat=fill;
		}
		return fillCercat;
	}


}
