package uoc.ei.exemples.modul4.recorreguts;

import uoc.ei.tads.*;

/** Aquesta classe defineix de manera recursiva els recorreguts bsics sobre
 * els arbres generals. Proporciona un tractament per defecte que consisteix
 * en mostrar per la sortida estndar els elements de l'arbre.<p>
 * En classes derivades es pot redefinir el mtode tractarPosicio amb l'objectiu
 * de redefinir el tractament aplicat a l'arbre.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 * @param <E>
 */

public abstract class RecorregutsArbre<E> {

	public RecorregutsArbre() {
		super();
	}


	public void preordre(Arbre<E> arbre) {
		inicialitzarRecorregut();
		if (!arbre.estaBuit())
			preordre(arbre,arbre.arrel());
		finalitzarRecorregut();
	}


	public void postordre(Arbre<E> arbre) {
		inicialitzarRecorregut();
		if (!arbre.estaBuit())
			postordre(arbre,arbre.arrel());
		finalitzarRecorregut();
	}


	public void perNivells(Arbre<E> arbre) {
		inicialitzarRecorregut();
		Cua<Posicio<E>> posicions=new CuaVectorImpl<Posicio<E>>(arbre.nombreElems());
		if (!arbre.estaBuit())
			posicions.encuar(arbre.arrel());
		while (!posicions.estaBuit()) {
			Posicio<E> p=posicions.desencuar();
			tractarPosicio(p);
			Recorregut<E> fills=arbre.fills(p);
			while (fills.hiHaSeguent())
				posicions.encuar(fills.seguent());
		}
		finalitzarRecorregut();
	}


	protected void preordre(Arbre<E> arbre,Posicio<E> p) {
		tractarPosicio(p);
		Recorregut<E> fills=arbre.fills(p);
		while (fills.hiHaSeguent()) {
			Posicio<E> fill=fills.seguent();
			preordre(arbre,fill);
		}
	}


	protected void postordre(Arbre<E> arbre,Posicio<E> p) {
		Recorregut<E> fills=arbre.fills(p);
		while (fills.hiHaSeguent()) {
			Posicio<E> fill=fills.seguent();
			postordre(arbre,fill);
		}
		tractarPosicio(p);
	}


	protected void tractarPosicio(Posicio<E> posicio) {
		System.out.print(posicio.getElem()+" ");
	}


	protected void inicialitzarRecorregut() {}


	protected void finalitzarRecorregut() {}


}
