package uoc.ei.exemples.modul8.academia;

import uoc.ei.tads.*;
import uoc.ei.tads.grafs.*;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class AcademiaElectricitatImpl implements AcademiaElectricitat {

	private Diccionari<String,AssignaturesAlumne> alumnes;
	private GrafAssignatures assignatures;
	private Diccionari<String,AssignaturesProfessor> professors;


	public AcademiaElectricitatImpl(GrafAssignatures assignatures,Iterador<String> profes) {
		alumnes=new DiccionariAVLImpl<String,AssignaturesAlumne>();
		this.assignatures=assignatures;
		professors=new DiccionariLlistaImpl<String,AssignaturesProfessor>();
		while (profes.hiHaSeguent()) {
			String professor=profes.seguent();
			professors.afegir(professor,new AssignaturesProfessor());
		}
	}


	public void afegirAlumne(String alumne) {
		AssignaturesAlumne assignatures=new AssignaturesAlumne();
		alumnes.afegir(alumne,assignatures);
	}


	public void matricular(String alumne, String assignatura) throws ExcepcioPrerequisitNoSatisfet {
		Vertex<Assignatura> vertexAssignatura=assignatures.consultarVertex(new Assignatura(assignatura,0));
		comprovarPrerequisits(alumne,vertexAssignatura);
		AssignaturesAlumne assignaturesAlumne=alumnes.consultar(alumne);
		assignaturesAlumne.novaAssignaturaEnCurs(vertexAssignatura);
	}


	public void assignaturaAprovada(String alumne, String assignatura,double qualificacio) {
		AssignaturesAlumne assignaturesAlumne=alumnes.consultar(alumne);
		Vertex<Assignatura> vertexAssignatura=assignatures.consultarVertex(new Assignatura(assignatura,0));
		assignaturesAlumne.assignaturaAprovada(vertexAssignatura,qualificacio);
	}


	public void imparteixAssignatura(String professor, String assignatura) {
		AssignaturesProfessor assignaturesProfessor=professors.consultar(professor);
		Vertex<Assignatura> vertexAssignatura=assignatures.consultarVertex(new Assignatura(assignatura,0));
		assignaturesProfessor.afegirAlFinal(vertexAssignatura);
	}


	public Iterador<? extends Nota> expedientAcademic(String alumne) {
		AssignaturesAlumne assignaturesAlumne=alumnes.consultar(alumne);
		return assignaturesAlumne.expedient();
	}


	public double carregaDocent(String professor) {
		AssignaturesProfessor assignaturesProfessor=professors.consultar(professor);
		return assignaturesProfessor.getCredits();
	}


	protected void comprovarPrerequisits(String alumne,Vertex<Assignatura> assignatura)
						throws ExcepcioPrerequisitNoSatisfet {
		AssignaturesAlumne assignaturesAprovades=alumnes.consultar(alumne);
		Iterador<Aresta<EtiquetaBuida,Assignatura>> arestesPrerequisits=assignatures.arestesAmbDestiA(assignatura);
		while (arestesPrerequisits.hiHaSeguent()) {
			ArestaDirigida<EtiquetaBuida,Assignatura> arestaD=(ArestaDirigida<EtiquetaBuida,Assignatura>)arestesPrerequisits.seguent();
			Vertex<Assignatura> prerequisit=arestaD.getVertexOrigen();
			if (!assignaturesAprovades.estaAprovada(prerequisit))
				throw new ExcepcioPrerequisitNoSatisfet(alumne,assignatura.getValor().getNom(),prerequisit.getValor().getNom());
		}


	}



}
