package uoc.ei.exemples.modul8.academia;

import uoc.ei.tads.*;
import uoc.ei.tads.grafs.*;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
class AssignaturesAlumne {

	private LlistaEncadenada<Vertex<Assignatura>> assignaturesEnCurs;
	private LlistaEncadenada<AssignaturaAprovada> assignaturesAprovades;


	public AssignaturesAlumne() {
		assignaturesEnCurs=new LlistaEncadenada<Vertex<Assignatura>>();
		assignaturesAprovades=new LlistaEncadenada<AssignaturaAprovada>();
	}


	public void novaAssignaturaEnCurs(Vertex<Assignatura> assignatura) {
		assignaturesEnCurs.afegirAlFinal(assignatura);
	}


	public void assignaturaAprovada(Vertex<Assignatura> assignatura,double qualificacio) {
		AssignaturaAprovada nota=new AssignaturaAprovada(assignatura,qualificacio);
		esborrarAssignaturaEnCurs(assignatura);
		assignaturesAprovades.afegirAlFinal(nota);
	}


	public Iterador<AssignaturaAprovada> expedient() {
		return assignaturesAprovades.elements();
	}


	public double mitjanaExpedient() {
		double sumaNotes=0;
		double nCredits=0;
		Iterador<AssignaturaAprovada> notes=expedient();
		while (notes.hiHaSeguent()) {
			Nota nota=notes.seguent();
			sumaNotes+=nota.getQualificacio()*nota.getAssignatura().getCredits();
			nCredits+=nota.getAssignatura().getCredits();
		}
		return sumaNotes/nCredits;
	}


	public boolean estaAprovada(Vertex<Assignatura> assignatura) {
		Iterador<AssignaturaAprovada> assignatures=expedient();
		boolean trobat=false;
		while (assignatures.hiHaSeguent() && !trobat) {
			AssignaturaAprovada aprovada=assignatures.seguent();
			trobat= assignatura==aprovada.getVertexAssignatura();
		}
		return trobat;
	}


	protected void esborrarAssignaturaEnCurs(Vertex<Assignatura> assignatura) {
		Recorregut<Vertex<Assignatura>> posicions=assignaturesEnCurs.posicions();
		Posicio<Vertex<Assignatura>> posicio=null;
		boolean trobat=false;
		while (posicions.hiHaSeguent() && !trobat) {
			posicio=posicions.seguent();
			trobat=posicio.getElem()==assignatura;
		}
		assignaturesEnCurs.esborrar(posicio);
	}


	public String toString() {
		return "{"+assignaturesEnCurs+";"+assignaturesAprovades+"}";
	}


}
