package uoc.ei.exemples.modul9.dns;


import uoc.ei.tads.*;

/**
 * Implementaci de la interfcie DNS usant un contenidor de TLD's (ContTLDs)
 * i un contenidor pels ms visitats (TopTen).
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class DNSimpl implements DNS {

  private DiccionariTLDs tlds;
  private TopTen topTen;

  /**
  * Mtode que inicialitza un nou objecte DNSimpl.
  */
  public DNSimpl() {
    tlds=new DiccionariTLDs(2001);
    topTen=new TopTen();
  }

  /**
  * Mtode que dna d'alta un domini relacionant-lo amb una ip.
  *
  * @param domini objecte Domini que cont el domini. ex: "www.uoc.edu".
  * @param ip  cadena que cont l'adrea ip del domini. ex: "129.34.2.3".
  */
  public void altaDomini(Domini d, IP ip) {
    TLD tld=d.getTLD();
    Entitat entitat=d.getEntitat();
    Host host=d.getHost();
    if (!tlds.hiEs(tld))
		tlds.afegir(tld,new DiccionariEntitats());
	DiccionariEntitats ets=tlds.consultar(tld);
    if (!ets.hiEs(entitat))
		ets.afegir(entitat,new DiccionariHosts());
    DiccionariHosts hosts=(DiccionariHosts)ets.consultar(entitat);
    if (hosts.hiEs(host)) {
    	InformacioHost h=hosts.consultar(host);
    	h.setIP(ip);
    } else {
        hosts.afegir(host,new InformacioHost(d,ip));
    }
  }


  /** Mtode que dna de baixa un domini.
  * @param domini objecte Domini que cont el domini. ex: "www.uoc.edu".
  */
  public void baixaDomini(Domini d) {
    TLD tld=d.getTLD();
    Entitat entitat=d.getEntitat();
    Host host=d.getHost();
    if (tlds.hiEs(tld)) {
		DiccionariEntitats ets=tlds.consultar(tld);
		if (ets.hiEs(entitat)) {
			DiccionariHosts hosts=ets.consultar(entitat);
			if (hosts.hiEs(host))
				hosts.esborrar(host);
		}
    }
  }


  /** Mtode que consulta l'adrea ip d'un domini.
   * @param domini objecte Domini que cont el domini. ex: "www.uoc.edu".
   * @return ip  returna una cadena que cont l'adrea ip del domini. ex: "129.34.2.3".
   */
  	public IP consultar(Domini d) {
		IP ip=null;
		TLD tld=d.getTLD();
		Entitat entitat=d.getEntitat();
		Host host=d.getHost();
		if (tlds.hiEs(tld)) {
			DiccionariEntitats ets=tlds.consultar(tld);
			if (ets.hiEs(entitat)) {
				DiccionariHosts hosts=ets.consultar(entitat);
				if (hosts.hiEs(host)) {
					InformacioHost infoHost=hosts.consultar(host);
					infoHost.visita();
					topTen.actualitza(infoHost);
					ip=infoHost.getIP();
				}
			}
		}
    return ip;
  }


  /** Mtode que consulta un dels dominis ms visitats.
  * @param num posici dins del ranking (0..9).
  * @return domini retorna un objecte Domini que cont el nom del domini.
  */
  public Domini mesVisitats(int n) {
	  Domini resultat=null;
	  InformacioHost info=topTen.consulta(n);
	  if (info!=null)
		  resultat=info.getDomini();
	  return resultat;
  }


  /** Mtode que inicialitza un tld per a fer un recorregut.
  * @param tld el tld del qual volem fer un recorregut.
  */
  public Iterador<Entitat> entitats(TLD t) {
	  Iterador<Entitat> res=null;
	  if (tlds.hiEs(t))
		  res=tlds.consultar(t).claus();
	  return res;
  }

}
