package uoc.ei.exemples.modul9.dns;


import java.util.StringTokenizer; // per extreure el tld, entitat i host d'una cadena

/**
 * Classe que conte la cadena que defineix un domini (ex: "www.uoc.edu").
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class Domini {
  private TLD tld;
  private Entitat entitat;
  private Host host;

  /**
  * Constructor d'un Domini a partir d'un TLD, una Entitat i un Host.
  * @param t cadena que identifica el TLD (ex "edu").
  * @param e cadena que identifica l'entitat (ex "uoc").
  * @param h cadena que identifica el host (ex "www").
  */
  public Domini(TLD t,Entitat e,Host h) {
    tld=t; entitat=e; host=h;
  }

  /**
  * Constructor d'un Domini a partir d'una cadena (ex "www.uoc.edu").
  * @param d cadena que cont el nom del domini (ex "www.uoc.edu").
  */
  public Domini(String d) {
  	StringTokenizer s=new StringTokenizer(d,".");
  	host=new Host(s.nextToken());
  	entitat=new Entitat(s.nextToken());
  	tld=new TLD(s.nextToken());
  }

  /**
  * Mtode que retorna el tld del domini.
  * @return TLD  el tld del domini (ex de "www.uoc.edu" retorn "edu").
  */
  public TLD getTLD() {return tld;}

  /**
  * Mtode que retorna l'entitatdel domini.
  * @return Entitat l'entitat del domini (ex de "www.uoc.edu" retorn "uoc").
  */
  public Entitat getEntitat() {return entitat;}

  /**
  * Mtode que retorna el host del domini.
  * @return Host  el host del domini (ex de "www.uoc.edu" retorn "www").
  */
  public Host getHost() {return host;}

  /**
  * Mtode que retorna una cadena amb el nom del domini.
  * @return String cadena amb el nom del domini (ex "www.uoc.edu").
  */
  public String toString() {
    return tld.toString()+"."+entitat.toString()+"."+host.toString();
  }
}