package uoc.ei.exemples.modul9.dns;

/**
 * Classe que implementa la llista dels top 10.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
class TopTen{
  InformacioHost t[];
  int n;

  /**
  * Constructor del top ten per a 10 dominis.
  */
  public TopTen() {
    t=new InformacioHost[10];
    n=0;
  }

  /**
  * Mtode que cerca un host (comparant el domini) dins dels top 10.
  * @param h informaci del host (cont domini, ip i nombre de visites).
  * @return int posici dins dels top 10 del host h.
  */
  private int cerca(InformacioHost h) {
    int pos=0;
    while (pos<n && t[pos]!=h) pos++;
    if (pos==n || t[pos]!=h) pos=-1;
    return pos;
  }

  /**
  * Mtode que reordena el host que ocupa la posici pos d'acord amb el nombre de visites.
  * @param pos posici del host que cal reordenar.
  */
  private void reordena(int pos) {
    while (pos-1>=0 && t[pos-1].getVisites()<t[pos].getVisites()) {
      InformacioHost tmp=t[pos];
      t[pos]=t[pos-1];
      t[pos-1]=tmp;
      pos--;
    }
  }

  /**
  * Mtode que actualitza la llista dels top 10 tenint en compte que el host h ha rebut una nova consulta.<br>
  * Si el host existeix als top 10 s'actualitza la seva posicio.<br>
  * Altrament, si la llista dels top 10 no est plena s'afegeix directament i,
  * en cas contrari, s'afegeix si t ms visites que el menys visitat dels top 10.
  * @param h informaci del host (cont domini, ip i nombre de visites).
  */
  public void actualitza(InformacioHost h) {
    int pos=cerca(h);
    if (pos>=0) { // si existeix
      reordena(pos);
    } else {
      if (n<10) t[n++]=h;
      else {
        if (t[9].getVisites()<h.getVisites()) {
          t[9]=h;
          reordena(9);
        }
      }
    }
  }

  /**
  * Mtode que retorna el host n ms visitat.
  * @param n nmero de host dins dels top ten.
  * @return ContHost informaci del host ms visitat nemro n.
  */
  public InformacioHost consulta(int n) { return t[n]; }

}
