package uoc.ei.tads;
/**
 * Interfcie que defineix les operacions de qualsevol arbre (tree).
 *
 * Els arbres sn estructures que relacionen els seus elements, anomenats
 * nodes, formant jerarquies: tot node (tret de l'arrel que es el cap de
 * la jerarquia) s descendent d'un node nic, i pot ser ascendent
 * d'altres nodes (quan no t descendents s'anomena fulla). Quan un node
 * pot tenir un nombre indeterminat de fills parlem d'arbres generals
 * (general tree) i, si en t un nombre fix N, d'arbres d'ordre N (n-ary
 * tree); en aquests ltims destaca el cas de N = 2, els anomenats arbres
 * binaris (binary tree).
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public interface Arbre<E>  extends Contenidor<E>
{
   /**
    * Accessor de lectura de l'arrel de l'arbre, si n'hi ha.
    * @return  arrel de l'arbre
    */
   public Posicio<E> arrel();

   /**
    * Mtode que soporta mltiples recorreguts, de les posicions filles
    * de la posici de referncia, simultanis i independents entre ells.
    * @param pare  posici de referncia
    * @return  enumeraci de les posicions filles
    */
   public Recorregut<E> fills(Posicio<E> pare);

   /**
    * Comprova si l'arbre o subarbre t algn fill.
    * @param pos  posici de referncia
    * @return  fals o cert, segons si t algun fill o no en t cap
    */
   public boolean esFulla(Posicio<E> pos);

   /**
    * Afegeix un element com a nou fill de la posici rebuda, si es pot.
    * Si el pare s null, s'afegeix a l'arrel; si s una fulla, s'afegeix
    * com a primer fill (fill ms a l'esquerra).
    * @param pare  posici de referncia
    * @param elem  element que es vol afegir a l'arbre
    * @return  nou fill; o l'arrel, si el pare era null
    */
   public Posicio<E> afegir(Posicio<E> pare,E elem);

   /**
    * Reemplaa l'element contingut a la posici rebuda.
    * @param elem  nou element
    * @param pos  posici de referncia
    * @return  element que hi havia a la posici
    */
   public E reemplacar(Posicio<E> pos, E elem);

   /**
    * Intercanvia en l'arbre els elements continguts a les
    * posicions rebudes. L'arbre resultant contindr els mateixos
    * elements que tenia, excepte pels dos elements rebuts com
    * a parmetre, que apareixeran intercanviats.<p>
    * El fet de si els objectes posici s'intercanvien, o b el que
    * s'intercanvia sn els elements continguts a les posicions, dependr
    * de cara implementaci concreta d'arbre.
    * @param pos1  primera de les dues posicions de referncia
    * @param pos2  segona de les dues posicions de referncia
    */
   public void intercanviar(Posicio<E> pos1, Posicio<E> pos2);

   /**
    * Esborra el subarbre representat per la posici fill, si es pot.
    * Si la posici del pare s null, esborra l'arbre sencer.
    * @param pare  posici del pare; pot ser null
    * @param fill  posici del fill
    */
   public void esborrar(Posicio<E> pare, Posicio<E> fill);

   /** Recorregut de les posicions de l'arbre.
    * @return  enumeraci de les posicions del contenidor per nivell
    */
   public Recorregut<E> posicions();

   /** Recorregut en preordre de les posicions de l'arbre.
    * @return  enumeraci de les posicions del contenidor preordre
    */
   public Recorregut<E> recorregutPreordre();

   /** Recorregut en postordre de les posicions de l'arbre.
    * @return  enumeraci de les posicions del contenidor postordre
    */
   public Recorregut<E> recorregutPostordre();


   /** Recorregut per nivells de les posicions de l'arbre.
    * @return  enumeraci de les posicions del contenidor postordre
    */
   public Recorregut<E> recorregutPerNivells();
}
