package uoc.ei.tads;
/**
 * Classe que emparella dos objectes. El primer (clau) ha d'identificar
 * la parella, el segon element (valor) emmagatzema una informaci
 * associada a la clau i pot ser qualsevol objecte.
 *
 * Implementa la interfcie Serializable per a poder convertir a cadenes o
 * fluxos de bytes (streams) els objectes del contenidor i gravar-los o
 * transmetre'ls.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class ClauValor<C,E>  implements java.io.Serializable, Comparable<ClauValor<C,E>>
{
   /** Atribut que en Java 1.5 determina la compatibilitat entre objectes
    * serialitzables de la mateixa classe. L'deitificador es calcula
    * mitjanant un mtode de la classe Utilitats. */
   private static final long serialVersionUID = Utilitats.getSerialVersionUID();
	
	
   /** Clau o ndex que permet identificar la parella. */
   protected C clau;

   /** Informaci associada a la clau. Pot tenir valor nul. */
   protected E valor;

   /**
    * Constructor amb dos parmetres.
    * @param clau  element comparable 
    * @param valor  informaci que ha d'emmagatzemar l'objecte
    * @pre clau!=null, new ExcepcioParametreIncorrecte("la clau ha de ser diferent de null")
    * @exception ExcepcioParametreIncorrecte  si la clau s null
    */
   public ClauValor(C clau, E valor)
   {
      this.clau = clau;
      this.valor = valor;
   }

   /**
    * Accessor de lectura de la clau o ndex.
    * @return  valor de la informaci associada a la clau
    */
   public C getClau() { return  clau; }

   /**
    * Accessor d'escriptura de la informaci associada a la clau.
    * @param valor  nou valor de la informaci emmagatzemada
    */
   public void setValor(E valor) { this.valor = valor; }

   /**
    * Accessor de lectura de la informaci associada a la clau.
    * @return  valor de la informaci emmagatzemada
    */
   public E getValor() { return  valor; }

   /**
    * Mtode que sobreescriu Object.hashCode(). Delega en el mateix mtode
    * implementat o heretat per la clau.
    * @return  codi de dispersi corresponent a la clau
    */
   public int hashCode() { return  clau.hashCode(); }

   /**
    * Mtode que sobreescriu Object.equals(Object obj). Delega en el
    * mateix mtode implementat o heretat per la clau.
    * @return  cert si l'objecte rebut com a parmetre s de la classe
    *          ClauValor i t la mateixa clau; altrament retorna fals
    */
   public boolean equals(Object clauValor)
   {
   	  if (clauValor==null) return false;
   	  return clau.equals( ((ClauValor)clauValor).getClau() );
   }

   
   /**
    * Mtode que implementa Comparable.compareTo(Object o). Delega en el
    * mateix mtode implementat o heretat per la clau.
    * @param clauValor  objecte de la mateixa classe (ClauValor)
    * @return  un enter negatiu, zero o positiu, segons si la clau
    *          d'aquest objecte (this.clau) s ms petita, igual o ms
    *          gran que la clau de l'objecte de la mateixa classe rebut com
    *          a parmetre (clauValor.clau)
    */
   public int compareTo(ClauValor<C,E> clauValor)
   {
      return  ((Comparable<C>)clau).compareTo(
                 (clauValor).getClau() );
   }

   
   /**
    * Mtode que redefineix la conversi de l'objecte a String per
    * facilitar la depuraci del codi.
    * @return  cadena de carcters amb la clau i el valor
    */
   public String toString() { return  ( "[" + clau + ": " + valor +"]"); }
}
