package uoc.ei.tads;
/**
 * Interfcie que defineix les operacions bsiques d'un conjunt.
 *
 * Els conjunts sn estructures que emmagatzemen elements no repetits. La
 * classe dels objectes ha de disposar d'una operaci d'igualtat.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public interface Conjunt<E>  extends Contenidor<E>
{
   /**
    * Afegeix un element, si es pot.
    * @param elem  element que es vol afegir al conjunt
    */
   public void afegir(E elem);

   /**
    * Comprova si hi ha un element.
    * @param elem  element de referncia
    * @return  cert o fals, segons si troba o no troba l'element
    */
   public boolean hiEs(E elem);

   /**
    * Esborra un element, si es pot.
    * @param elem  element de referncia
    * @return  element esborrat; o null, si no hi era
    */
   public E esborrar(E elem);
   
   /**
    * Afegeix els elements d'un segon conjunt que no existeixen en el
    * conjunt actual, si es pot. Si troba un element equivalent, segons la
    * funci de comparaci, el sobreescriu.
    * @param conj  conjunt que es vol unir a l'actual;
    *              pot ser buit, per no null
    * @exception ExcepcioParametreIncorrecte  si el conjunt s null
    */
   public void unio(Conjunt<E> conj);

   /**
    * Esborra del conjunt actual els elements que no existeixen en un
    * segon conjunt, si es pot.
    * @exception ExcepcioParametreIncorrecte  si el conjunt s null
    * @param conj  conjunt que es vol interseccionar amb l'actual;
    *              pot ser buit, per no null
    */
   public void interseccio(Conjunt<E> conj);

   /**
    * Esborra del conjunt actual els elements que existeixen en un segon
    * conjunt, si es pot.
    * @exception ExcepcioParametreIncorrecte  si el conjunt s null
    * @param conj  conjunt que es vol restar de l'actual;
    *              pot ser buit, per no null
    */
   public void diferencia(Conjunt<E> conj);
}
