
package uoc.ei.tads;

/** Clas que implementa aquelles operacions de conjunt que no
 * depenen de la representaci triada (AVL, Taules de dispersi, ...).
 * Aquestes operacions es corresponen amb les que tracten
 * amb subconjunts.
 * 
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */

public abstract class ConjuntAbstracte<E> implements Conjunt<E>
{

	/**
	* Afegeix els elements d'un segon conjunt que no existeixin en el
	* conjunt actual, si es pot. Si troba un element equivalent, segons la
	* funci de comparaci, el sobreescriu.
	* @param conj  conjunt que es vol unir a l'actual;
	*              pot ser buit, per no null
	* @exception ExcepcioParametreIncorrecte  si el conjunt s null
	* @pre conj!=null, new ExcepcioParametreIncorrecte("conjunt null");
	*/
	public void unio(Conjunt<E> conj) {
	   	Iterador<E> iter=conj.elements();
	   	while (iter.hiHaSeguent())
	   		afegir(iter.seguent());
	   }

	/**
	* Esborra del conjunt actual els elements que no existeixen en un
	* segon conjunt, si es pot.
	* @exception ExcepcioParametreIncorrecte  si el conjunt s null
	* @param conj  conjunt que es vol interseccionar amb l'actual;
	*              pot ser buit, per no null
	* @pre conj!=null, new ExcepcioParametreIncorrecte("conjunt null");
	*/
	public void interseccio(Conjunt<E> conj) {
	      Llista<E> elementsAEsborrar = new LlistaEncadenada<E>();
	      // En primer lloc guardem a una llista els elements que haurem
	      // d'esborrar del conjunt
	      Iterador<E> iter=elements();
	      while (iter.hiHaSeguent()) {
	      	E elem=iter.seguent();
	      	if (!conj.hiEs(elem))
	      		elementsAEsborrar.afegirAlFinal(elem);
	      }
	      // en segon lloc esborrem els elements
	      iter=elementsAEsborrar.elements();
	      while (iter.hiHaSeguent())
	      	esborrar(iter.seguent());
	   }

	/**
	* Esborra del conjunt actual els elements que existeixen en un segon
	* conjunt, si es pot.
	* @exception ExcepcioParametreIncorrecte  si el conjunt s null
	* @param conj  conjunt que es vol restar de l'actual;
	*              pot ser buit, per no null
	* @pre conj!=null, new ExcepcioParametreIncorrecte(conjunt null")
	*/
	public void diferencia(Conjunt<E> conj) {
	   	Iterador<E> iter=conj.elements();
	   	while (iter.hiHaSeguent())
	   		esborrar(iter.seguent());
	   }

}
