package uoc.ei.tads;

import java.io.Serializable;

/**
 * Interfcie que defineix les operacions de qualsevol TAD que es
 * caracteritzi per contenir un conjunt d'elements. No es contemplen els
 * mtodes crea() ni destrueix(): les classes que la implementin poden
 * definir-los o deixar que el compilador faci servir el constructor per
 * defecte, i el "garbage collector" per als objectes no referenciats.
 *
 * Tamb defineix, a efectes prctics, el mtode elements() que permet
 * visualitzar tots els elements del contenidor mitjanant un iterador.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public interface Contenidor<E> extends Serializable
{
   /**
    * Comprova si el contenidor est buit.
    * @return  cert si el contenidor est buit
    */
   public boolean estaBuit();

   /**
    * Accessor de lectura del nombre d'elements que hi ha al contenidor.
    * @return  nombre d'elements que hi ha al contenidor
    */
   public int nombreElems();

   /**
    * Accessor de lectura dels elements que hi ha al contenidor.
    * Retorna una enumeraci. Es pot obtenir un llistat amb un parell de
    * lnies de codi: <PRE>
    * for ( Iterador it = tad.elements(); it.hiHaSeguent(); )
    *    System.out.println(it.seguent()); </PRE>
    * Enumerar s simplement enunciar l'una darrere l'altra (les coses
    * d'una srie, les parts d'un tot). Per si el contenidor t definit
    * algun tipus d'ordenaci o de recorregut, l'enumeraci ha de ser
    * conseqent i oferir els elements per ordre (FIFO, LIFO, inordre,
    * etc.), sense alterar l'estat actual del contenidor.
    * @see  Iterador#hiHaSeguent()
    * @see  Iterador#seguent()
    * @exception ExcepcioPosicioInvalida  si es vol obtenir el segent
    *            element de l'enumeraci i no n'hi ha cap o no n'hi ha cap
    *            ms
    * @return  enumeraci dels elements del contenidor
    */
   public Iterador<E> elements();
}
