package uoc.ei.tads;
/**
 * Interfcie que defineix les operacions d'un diccionari.
 *
 * Els diccionaris sn estructures que emmagatzemen elements amb una clau
 * associada. La clau ha de disposar d'una operaci d'igualtat. En el cas
 * dels diccionaris ordenats ha de permetre l'ordenaci total entre claus.
 * Algunes implementacions permeten claus repetides i altres no. L'element
 * associat a la clau pot ser qualsevol objecte.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public interface Diccionari<C,E>  extends Contenidor<E>
{
   /**
    * Afegeix un element amb una clau associada, si es pot.
    * @param clau  clau associada a l'element que es vol afegir
    * @param elem  element que es vol afegir al diccionari
    */
   public void afegir(C clau, E elem);

   /**
    * Comprova si hi ha un element amb una determinada clau.
    * @param clau  clau associada a un element
    * @return  cert o fals, segons si troba o no troba la clau
    */
   public boolean hiEs(C clau);

   /**
    * Accesor de lectura de l'element associat amb una clau.
    * @param clau  clau de referncia
    * @return  element associat amb la clau
    */
   public E consultar(C clau);

   /**
    * Esborra la primera clau coincident i l'element associat, si es pot.
    * @param clau  clau de referncia
    * @return  element associat amb la clau que s'ha esborrat
    */
   public E esborrar(C clau);

   /**
    * Accessor de lectura dels elements que hi ha al contenidor.
    * Retorna una enumeraci. Es pot obtenir un llistat amb un parell de
    * lnies de codi: <PRE>
    * for ( Iterador it = tad.elements(); it.hiHaSeguent(); )
    *    System.out.println(it.seguent()); </PRE>
    * Enumerar s simplement enunciar l'una darrere l'altra (les coses
    * d'una srie, les parts d'un tot). Per si el contenidor t definit
    * algun tipus d'ordenaci o de recorregut, l'enumeraci ha de ser
    * conseqent i oferir els elements per ordre (FIFO, LIFO, inordre,
    * etc.), sense alterar l'estat actual del contenidor.
    * @see  Iterador#hiHaSeguent()
    * @see  Iterador#seguent()
    * @exception ExcepcioPosicioInvalida  si es vol obtenir el segent
    *            element de l'enumeraci i no n'hi ha cap o no n'hi ha cap
    *            ms
    * @return  enumeraci de les claus del contenidor
    */
   public Iterador<C> claus();
}
