/*
 */
package uoc.ei.tads;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class DiccionariLlistaImpl<C,E> implements Diccionari<C,E> {


	protected LlistaEncadenada<ClauValor<C,E>> diccionari;


	// Classes internes per a implementacio

	protected static class RecorregutAmbAnterior<E> implements Recorregut<E> {

		// Atributs

		Recorregut<E> recorregutReal;
		Posicio<E> act;
		Posicio<E> ant;


		// Constructor

		public RecorregutAmbAnterior(Recorregut<E> r) {
			recorregutReal=r;
			ant=null;
			act=null;
		}


		// Implementacio de metodes de Recorregut

		public Posicio<E> seguent() {
			ant=act;
			act=recorregutReal.hiHaSeguent() ? recorregutReal.seguent() : null;
			return act;
		}

		public boolean hiHaSeguent() {
			return recorregutReal.hiHaSeguent();
		}


		// Metode addicional que ens retorna l'anterior

		public Posicio<E> anterior() { return ant; }
		public Posicio<E> actual() { return act; }

	}


	// Constructors

	public DiccionariLlistaImpl() {
		diccionari=new LlistaEncadenada<ClauValor<C,E>>();
	}


	// Metodes d'implementaci

	protected RecorregutAmbAnterior<ClauValor<C,E>> cercaPosicio(C clau) {
		RecorregutAmbAnterior<ClauValor<C,E>> r=new RecorregutAmbAnterior<ClauValor<C,E>>(diccionari.posicions());
		boolean trobat=false;
		while (!trobat && r.hiHaSeguent()) {
			ClauValor<C,E> parellClauValor=r.seguent().getElem();
			trobat=clau.equals(parellClauValor.getClau());
		}
		if (!trobat)
			r.seguent();
		return r;
	}


	// Implementaci dels mtodes de Diccionari

	public void afegir(C clau,E obj) {
		if (!hiEs(clau))
			diccionari.afegirAlPrincipi(new ClauValor<C,E>(clau,obj));
	}


	public boolean hiEs(C clau) {
		RecorregutAmbAnterior<ClauValor<C,E>> r=cercaPosicio(clau);
		return r.actual()!=null && clau.equals(r.actual().getElem().getClau());
	}


	public E consultar(C clau) {
		RecorregutAmbAnterior<ClauValor<C,E>> r=cercaPosicio(clau);
		Posicio<ClauValor<C,E>> actual=r.actual();
		return actual!=null ? actual.getElem().getValor() : null;
	}


	public Iterador<C> claus() {
		return new IteradorRecorregutClausImpl<C,E>(diccionari.posicions());
	}


	public E esborrar(C clau) {
		E elementEsborrat=null;
		RecorregutAmbAnterior<ClauValor<C,E>> r=cercaPosicio(clau);
		Posicio<ClauValor<C,E>> actual=r.actual();
		if (actual!=null && clau.equals(actual.getElem().getClau())) {
			elementEsborrat=actual.getElem().getValor();
			if (r.anterior()==null)
				diccionari.esborrarPrimer();
			else
				diccionari.esborrarSeguent(r.anterior());
		}
		return elementEsborrat;
	}


	public boolean estaBuit() {
		return diccionari.estaBuit();
	}


	public int nombreElems() {
		return diccionari.nombreElems();
	}


	public Iterador<E> elements() {
		return new IteradorRecorregutValorsImpl<C,E>(diccionari.posicions());
	}


	public String toString() {
		return Utilitats.toStringContenidorDelegacio("DiccionariLlistaImpl",diccionari);
	}

}
