/*
 */
package uoc.ei.tads;

/** Implementaci d'un diccionari mitjanant un vector de mida fixa.
 * Aquesta implementaci concreta est pensada per a casos molt concrets
 * en que per una banda es requereix un diccionari on, un cop inicialitzat,
 * tindrem sempre el mateix nombre d'elements (conegut d'entrada); i per
 * altra banda, necessitem accs al vector de parell claus valor.
 * Concretament s'usa en l'algorisme de clcul dels camins mnims d'un graf
 * que proporciona la llibreria.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class DiccionariVectorImpl<C,E> implements Diccionari<C,E> {


	protected ClauValor<C,E>[] diccionari;
	protected int n;


	// Constructors

	public DiccionariVectorImpl(int n) {
		diccionari=(ClauValor<C,E>[])new ClauValor[n];
	}


	// funcionalitats extra proporcionades per aquesta implementaci de diccionari

	public ClauValor<C,E>[] getVector() { return diccionari; }


	// Implementaci dels mtodes de Diccionari

	public void afegir(C clau,E obj) {
		int i;
		for(i=0;i<n && !diccionari[i].getClau().equals(clau);i++);
		diccionari[i]=new ClauValor<C,E>(clau,obj);
		if (i==n)
			n++;
	}


	public E esborrar(C clau) {
		E elem=null;
		int i;
		for(i=0;i<n && !diccionari[i].getClau().equals(clau);i++);
		if (i<n) {
			elem=diccionari[i].getValor();
			diccionari[i]=diccionari[n-1];
			diccionari[n-1]=null;
			n--;
		}
		return elem;
	}


	public boolean hiEs(C clau) {
		int i;
		for(i=0;i<n && !diccionari[i].getClau().equals(clau);i++);
		return i<n;
	}


	public E consultar(C clau) {
		int i;
		for(i=0;i<n && !diccionari[i].getClau().equals(clau);i++);
		return i<n ? diccionari[i].getValor() : null;
	}


	public boolean estaBuit() {
		return n==0;
	}


	public int nombreElems() {
		return n;
	}


	public Iterador<C> claus() {
		return new IteradorClaus<C,E>(new IteradorVectorImpl<ClauValor<C,E>>(diccionari,n,0));
	}


	public Iterador<E> elements() {
		return new IteradorValors<C,E>(new IteradorVectorImpl<ClauValor<C,E>>(diccionari,n,0));
	}


	public String toString() {
		StringBuffer sb=new StringBuffer("{DiccionariVectorImpl: ");
		int i=0;
		while(i<n) {
			sb.append(diccionari[i]);
			i++;
			if (i<n) sb.append(", ");
		}
		sb.append("}");
		return sb.toString();
	}


	protected static class IteradorClaus<CI,EI> implements Iterador<CI> {

		private Iterador<ClauValor<CI,EI>> iteradorClauValor;

		public IteradorClaus(Iterador<ClauValor<CI,EI>> iterador) {
			iteradorClauValor=iterador;
		}

		public boolean hiHaSeguent() {
			return iteradorClauValor.hiHaSeguent();
		}

		public CI seguent() {
			if (!hiHaSeguent())
				throw new ExcepcioPosicioInvalida();
			return iteradorClauValor.seguent().getClau();
		}

	}


	protected static class IteradorValors<CI,EI> implements Iterador<EI> {

		private Iterador<ClauValor<CI,EI>> iteradorClauValor;

		public IteradorValors(Iterador<ClauValor<CI,EI>> iterador) {
			iteradorClauValor=iterador;
		}

		public boolean hiHaSeguent() {
			return iteradorClauValor.hiHaSeguent();
		}

		public EI seguent() {
			if (!hiHaSeguent())
				throw new ExcepcioPosicioInvalida();
			return iteradorClauValor.seguent().getValor();
		}

	}


}
