package uoc.ei.tads;
/**
 * Classe que implementa les operacions de recorregut bidireccional dels
 * elements d'un contenidor, per delegaci en un recorregut de posicions.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
class IteradorRecorregutBidireccionalImpl<ERes,ERec>  extends IteradorRecorregutImpl<ERes,ERec>
      implements IteradorBidireccional<ERes>
{
   /** Atribut que en Java 1.5 determina la compatibilitat entre objectes
    * serialitzables de la mateixa classe. L'deitificador es calcula
    * mitjanant un mtode de la classe Utilitats. */
   private static final long serialVersionUID = Utilitats.getSerialVersionUID();
	
	
   /**
    * Constructor amb un parmetre.
    * @param recorregut  recorregut que proporciona els elements
    */
   public IteradorRecorregutBidireccionalImpl(Recorregut<ERec> recorregut)
   {
      super(recorregut);
   }

   /**
    * Comprova si hi ha un darrer o anterior element. s sensible a
    * eventuals alteracions de l'estructura de posicions. Retorna fals si
    * el contenidor est buit o ja s'ha visitat el primer element.
    * @return  cert o fals, segons si es pot recular o no es pot
    */
   public boolean hiHaAnterior()
   {
      return  ((RecorregutBidireccional<ERec>)recorregut).hiHaAnterior();
   }

   /**
    * Accessor de lectura del darrer o anterior element de l'enumeraci.
    * @exception ExcepcioPosicioInvalida  si es vol obtenir l'anterior
    *            element de l'enumeraci i no n'hi ha cap o no n'hi ha cap
    *            ms
    * @return  darrer o anterior element a l'actual
    */
   public ERes anterior()  throws  ExcepcioPosicioInvalida
   {
   	  Posicio<ERec> posicio=((RecorregutBidireccional<ERec>)recorregut).anterior();
      return (ERes)getElem(posicio);
   }
}
