package uoc.ei.tads;
/**
 * Classe que implementa les operacions de recorregut dels elements d'un
 * contenidor, per delegaci en un recorregut de posicions.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class IteradorRecorregutImpl<ERes,ERec>  implements Iterador<ERes>
{
   /** Atribut que en Java 1.5 determina la compatibilitat entre objectes
    * serialitzables de la mateixa classe. L'deitificador es calcula
    * mitjanant un mtode de la classe Utilitats. */
   private static final long serialVersionUID = Utilitats.getSerialVersionUID();
	
	
   /** Recorregut que implementa les operacions per delegaci. */
   protected Recorregut<ERec> recorregut;

   /**
    * Constructor amb un parmetre.
    * @param recorregut  recorregut que proporciona els elements
    */
   public IteradorRecorregutImpl(Recorregut<ERec> recorregut)
   {
      this.recorregut = recorregut;
   }

   /**
    * Comprova si hi ha un primer o segent element. s sensible a
    * eventuals alteracions de l'estructura de posicions. Retorna fals si
    * el contenidor est buit o ja s'ha visitat l'ltim element.
    * @return  cert o fals, segons si es pot avanar o no es pot
    */
   public boolean hiHaSeguent() { return  recorregut.hiHaSeguent(); }

   /**
    * Accessor de lectura del primer o segent element de l'enumeraci.
    * @exception ExcepcioPosicioInvalida  si es vol obtenir el segent
    *            element de l'enumeraci i no n'hi ha cap o no n'hi ha cap
    *            ms
    * @return  primer o segent element a l'actual
    */
   public ERes seguent()  throws  ExcepcioPosicioInvalida
   {
      return getElem(recorregut.seguent());
   }
 
   /**
    * Accessor de lectura de l'element emmagatzemat a la posici.
    * @see  IteradorRecorregutImpl#seguent()    
    * @param pos  posici que cont l'element
    * @return  element contingut a la posici
    * @pre pos!=null, ExcepcioPosicioInvalida
    */
   protected ERes getElem(Posicio<ERec> pos) {
   	return  (ERes)pos.getElem();
   }
   
}
