package uoc.ei.tads;

import uoc.ei.tads.LlistaEncadenada.RecorregutUnidireccionalLLista;

/**
 * Seqncia posicional que es caracteritza per disposar d'operacions
 * basades en la posici que ocupa un element dins del contenidor.
 *
 * @see  Posicio
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public interface Llista<E>  extends Contenidor<E>
{
   /**
    * Afegeix un element al principi de la llista.
    * @param elem  element que es vol afegir a la llista
    * @return  nova posici que cont l'element
    */
   public Posicio<E> afegirAlPrincipi(E elem);

   /**
    * Afegeix un element al final de la llista.
    * @param elem  element que es vol afegir a la llista
    * @return  nova posici que cont l'element
    */
   public Posicio<E> afegirAlFinal(E elem);

   /**
    * Afegeix un element abans de la posici rebuda.
    * @param elem  element que es vol afegir a la llista
    * @param pos  posici de referncia
    * @exception ExcepcioPosicioInvalida  si la posici s null o no
    *            vlida
    * @return  nova posici que cont l'element
    */
   public Posicio<E> afegirAbansDe(Posicio<E> pos, E elem);

   /**
    * Afegeix un element desprs de la posici rebuda.
    * @param elem  element que es vol afegir a la llista
    * @param pos  posici de referncia
    * @exception ExcepcioPosicioInvalida  si la posici s null o no
    *            vlida
    * @return  nova posici que cont l'element
    */
   public Posicio<E> afegirDespresDe(Posicio<E> pos, E elem);

   /**
    * Esborra la primera posici de la llista.
    * @exception ExcepcioContenidorBuit  si la llista est buida
    * @return  element que hi havia a la posici
    */
   public E esborrarPrimer();

   /**
    * Esborra la posici rebuda.
    * @param pos  posici que es vol eliminar
    * @exception ExcepcioPosicioInvalida  si la posici s null o no
    *            vlida
    * @return  element que hi havia a la posici
    */
   public E esborrar(Posicio<E> pos);

   /**
    * Esborra la posici segent.
    * @param node  posici anterior a la que es vol eliminar
    * @exception ExcepcioPosicioInvalida  si la posici s null o no
    *            vlida
    * @return  element que hi havia a la posici segent
    */
   public E esborrarSeguent(Posicio<E> node);

   /**
    * Reemplaa l'element contingut a la posici rebuda.
    * @param elem  nou element
    * @param pos  posici de referncia
    * @exception ExcepcioPosicioInvalida  si la posici s null o no
    *            vlida
    * @return  element que hi havia a la posici
    */
   public E reemplacar(Posicio<E> pos, E elem);

   /**
    * Intercanvia els elements continguts a les posicions rebudes.
    * @param pos1  primera de les dues posicions de referncia
    * @param pos2  segona de les dues posicions de referncia
    * @exception ExcepcioPosicioInvalida  si la posici s null o no
    *            vlida
    */
   public void intercanviar(Posicio<E> pos1, Posicio<E> pos2);
   

   /**
    * @return Un recorregut sobre les posicions de la llista.
    */
   public Recorregut<E> posicions();

}
