package uoc.ei.tads;
/**
 * Classe que representa una posici en un contenidor implementat
 * mitjanant una representaci amb vector. T la utilitat de:
 * (a) A partir d'un objecte de tipus Rang, la implementaci del
 * contenidor pot accedir en temps constant a la posici del vector
 * que l'emmagatzema; i
 * (b)poder canviar elements de d'ubicaci dins del contenidor
 * sense invalidar referncies externes a elements (de tipus Posicio/Rang)
 * que s'hagin creat amb anterioritat.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class Rang<E>  implements Posicio<E> {

   /** Atribut que en Java 1.5 determina la compatibilitat entre objectes
    * serialitzables de la mateixa classe. L'deitificador es calcula
    * mitjanant un mtode de la classe Utilitats. */
   private static final long serialVersionUID = Utilitats.getSerialVersionUID();
	
	
	/** Element contingut al node. */
   private E element;

   /** ndex de l'element dins del contenidor. */
   private int index;

   /**
    * Constructor amb dos parmetres.
    * @param idx  ndex de la posici dins del contenidor.
    * @param elem  valor de l'element contingut a la posici.
    */
   public Rang(int idx, E elem) {
      index = idx;
      element = elem;
   }

   /**
    * Accessor d'escriptura del valor contingut a la posici.
    * @param elem  nou valor de l'element contingut al node
    */
   public void setElem(E elem) { element = elem; }

   
   /**
    * Accessor de lectura de l'element contingut al node.
    * @return  element contingut al node
    */
   public E getElem() { return  element; }

   
   /**
    * Accessor d'escriptura de l'ndex de la posici.
    * @param idx ndex de la posici dins del contenidor.
    */
   public void setIndex(int idx) { index = idx; }

   
   /**
    * Accesor de lectura de l'ndex de la posici.
    * @return  enter no negatiu
    */
   public int getIndex() { return  index; }

   
   /**
    * Mtode que redefineix la conversi de l'objecte a String per
    * facilitar la depuraci del codi. Delega en el mtode toString() de
    * l'element emmagatzemat a la posici.
    * @return  cadena de carcters representativa de l'element
    */
   public String toString()
   {
      return  (element == null) ? ("[" + index + ",null]") :
              ("[" + index + ","+ element.toString()+"]");
   }
}
