
package uoc.ei.tads;

import java.io.IOException;
import java.io.InputStream;

/** Aquesta classe proporciona un conjunt de mtodes esttics
 * que es fan servir a la implementaci dels TADs.
 * Tamb proporciona mtodes pblics esttics amb l'objectiu
 * de que es puguin fer servir per els usuaris de la
 * llibreria de TADs.
 * 
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class Utilitats {

	/** Versi actual de la llibreria de TADs. Es fa servir per a
	 * dotar d'un identificador a cada una de les classes de la
	 * llibreria. Aquest identificador es fa servir en la versi
	 * 1.5 de Java per a determinar la compatibilitat entre objectes
	 * serialitzats.
	 */
	private static final String versio = "2.00.00";
	

	/** Retorna la versi de la llibreria de TADs.
	 * @return La versi. El format s x.yy.zz. Un canvi en la 'x'
	 * significa un canvi en el disseny de la llibreria com a tal.
	 * Un canvi en 'yy' significa un canvi a nivell de funcionalitat;
	 * i un canvi a 'zz' significa un canvi on s'han solucionat
	 * problemes existents a la versi anterior.
	 */
	public static String getVersio() {
		return versio;
	}
	
	
	/** Retorna la versi de la llibreria de TADs com un long.
	 * @return el nombre corresponent a la versi de la llibreria.
	 * El mtode usat per a la conversi s: versi 1.00.00 --> 10000.
	 */
	public static long getVersioLong() {
		String str=versio.replaceAll("[.]","");
		return Long.parseLong(str);
	}
	
	/** Retorna un identificador que es pot fer servir per les classes
	 * de la llibreria per a determinar la compatibilitat entre objectes
	 * serialitzats de la mateixa classe. Actualment aquest identificador
	 * es correspon amb la versi de la llibreria.
	 * @return Identificador per a la serialitzaci.
	 */
	public static long getSerialVersionUID() {
		return getVersioLong();
	}

	
	/** Llegeix una lnia del stream d'entrada donat.
	 * @param str String d'informaci que es treu per la sortida estndar
	 * (til sobretot si l'entrada s l'entrada estndar per a treure un
	 * missatge del qu es demana. Si no, es pot deixar buit.
	 * @param in Stream d'entrada d'on es llegir la lnia.
	 * @return Un String amb la dada llegida.
	 * @throws IOException En cas que s'hagi produt algun problema
	 * al llegir la dada o treure el missatge per la sortida estndar.
	 */
	
    public static String llegirString(String str,InputStream in) throws IOException {
        if (str!=null) System.out.println(str);
        StringBuffer buffer=new StringBuffer();
        int c=in.read();
        while (c=='\n' || c=='\r') c=in.read();
        while (c!='\n' && c!='\r') {
            buffer.append((char)c);
            c=in.read();
        }
        return buffer.toString();
    }
    
    
	/** Retorna en un string una representaci textual del contenidor.
	 * @param nomClasse Nom de la classe corresponent al contenidor.
	 * @param rec Recorregut amb tots els elements del contenidor.
	 * @return un String amb la representaci textual del contenidor.
	 */
	
	public static String toStringContenidor(String nomClasse,Recorregut rec) {
	      StringBuffer buffer = new StringBuffer("{"+nomClasse+": ");
	      while (rec.hiHaSeguent()) {
	      	Posicio kv=rec.seguent();
	        buffer.append(kv.getElem());
	        if (rec.hiHaSeguent()) buffer.append(", ");
	      }
	      buffer.append("}");
	      return  buffer.toString();
		}


	/** Retorna en un string una representaci textual d'un contenidor que delega
	 * la seva implementaci en un altre contenidor (per exemple ConjuntAVLImpl
	 * delega la seva implementaci en ArbreAVL).
	 * @param nomClasse Nom de la classe corresponent al contenidor que fa
	 * servir delegaci.
	 * @param implementacio Contenidor Implementaci feta servir.
	 * @return un String amb la representaci textual del contenidor.
	 */
	
	public static String toStringContenidorDelegacio(String nomClasse,Contenidor implementacio) {
	   	return  "["+nomClasse+": "+implementacio.toString()+"]";
		}

	
}
