package uoc.ei.tads.algorismes.grafs;

import uoc.ei.tads.*;
import uoc.ei.tads.grafs.*;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class CaminsMinims<TElem,TEtiqueta extends Number> {


	protected static class VCM<E> {
		private Vertex<E> vertex;
		private double distancia;
		private boolean jaTenimDistanciaMinima;

		public VCM(Vertex<E> vertex,double distancia) {
			this.vertex=vertex;
			this.distancia=distancia;
			this.jaTenimDistanciaMinima=false;
		}

		public void jaTenimDistanciaDefinitiva() { jaTenimDistanciaMinima=true; }
		public boolean tenimJaDistanciaDefinitiva() { return jaTenimDistanciaMinima; }
		public double getDistancia() { return distancia; }
		public Vertex<E> getVertex() { return vertex; }
		public void setDistancia(double distancia) { this.distancia=distancia; }

		public String toString() { return "["+vertex+","+distancia+","+jaTenimDistanciaMinima+"]"; }

	}


	public ClauValor<Vertex<TElem>,Number>[] calcular(GrafDirigit<TElem,TEtiqueta> graf, Vertex<TElem> vertex) {
		VCM<TElem>[] d=(VCM<TElem>[])new VCM[graf.numVertexs()];
		int vertexsAmbDistanciaDefinitiva=0;
		Iterador<Vertex<TElem>> vertexs=graf.vertexs();
		// calculem distncies incials
		for(int i=0;vertexs.hiHaSeguent();i++) {
			Vertex<TElem> v=vertexs.seguent();
			d[i]=new VCM<TElem>(v,distanciaDirecta(graf,vertex,v));
			if (d[i].getDistancia()==0) {
				d[i].jaTenimDistanciaDefinitiva();
				vertexsAmbDistanciaDefinitiva++;
			}
		}
		while (vertexsAmbDistanciaDefinitiva<graf.numVertexs()-1) {
			int posicioW=triarVertexMesProper(graf,d);
			d[posicioW].jaTenimDistanciaDefinitiva();
			vertexsAmbDistanciaDefinitiva++;
			// actualitzem les distancies comprovant si mitjanant
			// w arribem abans
			double distanciaAW=d[posicioW].getDistancia();
			for(int i=0;i<d.length;i++) {
				if (!d[i].tenimJaDistanciaDefinitiva()) {
					double distanciaDesdeW=distanciaDirecta(graf,d[posicioW].getVertex(),d[i].getVertex());
					if (distanciaAW+distanciaDesdeW<d[i].getDistancia())
						d[i].setDistancia(distanciaAW+distanciaDesdeW);
				}
			}
		}
		ClauValor<Vertex<TElem>,Number>[] resultat=(ClauValor<Vertex<TElem>,Number>[])new ClauValor[d.length];
		for(int i=0;i<d.length;i++)
			resultat[i]=new ClauValor<Vertex<TElem>,Number>(d[i].getVertex(),d[i].getDistancia());
		return resultat;
	}


	private int triarVertexMesProper(GrafDirigit<TElem,TEtiqueta> graf,VCM<TElem>[] distanciesActuals) {
		double minim=Double.POSITIVE_INFINITY;
		int posicioW=-1;
		// triem vertex amb menor distancia directa que no es a s
		for(int i=0;i<distanciesActuals.length;i++)
			if (!distanciesActuals[i].tenimJaDistanciaDefinitiva())
				if (distanciesActuals[i].getDistancia()<minim) {
					minim=distanciesActuals[i].getDistancia();
					posicioW=i;
				}
		return posicioW;
	}


	private double distanciaDirecta(GrafDirigit<TElem,TEtiqueta> graf,Vertex<TElem> origen,Vertex<TElem> desti) {
		double valor;
		if (origen==desti)
			valor=0;
		else {
			Aresta<TEtiqueta,TElem> aresta=graf.obtenirAresta(origen,desti);
			if (aresta!=null)
				valor=aresta.getEtiqueta().doubleValue();
			else
				valor=Double.POSITIVE_INFINITY;
		}
		return valor;
	}

}
