package uoc.ei.tads.grafs;

/**
 * Classe que representa l'aresta d'un graf (dirigit o no dirigit).
 *
 * @author  Jordi Alvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
abstract class ArestaAbstracta<TEtiqueta,TElem> implements Aresta<TEtiqueta,TElem>
{
   /** Primer vrtex de l'aresta. En el cas d'una aresta dirigida, representa
    * l'origen. */
   protected Vertex<TElem> vertexA;

   /** Segon vrtex de l'aresta. En el cas d'una aresta dirigida representa
    * el dest. */
   protected Vertex<TElem> vertexB;

   /** Valor de l'etiqueta de l'aresta, o null si la relaci s no valorada. */
   protected TEtiqueta etiqueta;


   /**
    * Constructor amb dos paràmetres. No es permeten bucles.
    * @param vertex1  primer vrtex de l'aresta (o origen en el cas de dirigida)
    * @param vertex2  segon vrtex de l'aresta (o dest en el cas de dirigida)
    * @pre vertex1!=null && vertex2!=null, ExcepcioParametreIncorrecte
    */
   public ArestaAbstracta(Vertex<TElem> vertex1, Vertex<TElem> vertex2)
   {
      this.vertexA = vertex1;
      this.vertexB = vertex2;
   }


   /**
    * Accessor de lectura del valor associat a l'aresta.
    * @return  valor de l'aresta; o null, si s una relació no valorada
    */
   public TEtiqueta getEtiqueta() { return etiqueta; }

   
   /**
    * Accessor d'escriptura del valor associat a l'aresta.
    * @pre true
    * @param valor  valor de la relació
    * @post getValor()@pre == valor
    */
   public void setEtiqueta(TEtiqueta etiqueta) { this.etiqueta = etiqueta; }
  

   /**
    * Indica si l'aresta incideix en un vrtex donat, com a origen o
    * destí.
    * @param v  vrtex del qual es vol saber si pertany a l'aresta
    * @pre v != null
    * @post true
    * @return  cert si l'aresta incideix en el vrtex; fals altrament
    * @pre v!=null, ExcepcioParametreIncorrecte
    */
   public boolean incideixEnVertex(Vertex<TElem> vertex)
   {
      return ( this.vertexA.equals(vertex) || this.vertexB.equals(vertex) );
   }


   /**
    * Mtode que sobreescriu Object.equals(Object obj). Delega en els
    * mateixos mtodes implementats o heretats pels vrtexs.
    * @return  cert si l'objecte rebut com a parmetre s de la classe
    *          ArestaDirigida i t el mateixos vrtexs en el mateix ordre;
    *          altrament retorna fals
    */
   public boolean equals(Object object)
   {
   	  if (getClass()!=object.getClass())
   	  	return false;
   	  ArestaAbstracta<TEtiqueta,TElem> aresta=(ArestaAbstracta<TEtiqueta,TElem>)object;
   	  return vertexA.equals(aresta.vertexA) &&
   	  		 vertexB.equals(aresta.vertexB);
   }


   /**
    * Mtode que sobreescriu Object.toString(). Delega en els mateixos
    * mtodes implementats o heretats pels vrtexs i per l'etiqueta (si
    * n'hi ha).
    * @return  cadena de carcters amb el primer vrtex, l'etiqueta si en
    *          t i el segon vrtex
    */
   public String toString()
   {
   	  return "("+vertexA+":"+etiqueta+":"+vertexB+")";
   }
}
