package uoc.ei.tads.grafs;

import uoc.ei.tads.ExcepcioParametreIncorrecte;
import uoc.ei.tads.Utilitats;

/**
 * Classe que representa una aresta d'un graf dirigit. La relaci s entre
 * dos vrtexs del mateix domini i va del vrtex origen al de dest.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
class ArestaDImpl<TEtiqueta,TElem>  extends ArestaImpl<TEtiqueta,TElem>
									implements ArestaDirigida<TEtiqueta,TElem>
{
   /** Atribut que en Java 1.5 determina la compatibilitat entre objectes
    * serialitzables de la mateixa classe. L'deitificador es calcula
    * mitjanant un mtode de la classe Utilitats. */
   private static final long serialVersionUID = Utilitats.getSerialVersionUID();


   /** Constructor amb dos parmetres.
    * @param origen  vrtex origen de l'aresta
    * @param desti  vrtex dest de l'aresta
    * @exception ExcepcioParametreIncorrecte  si algun vrtex s null
    * @pre origen!=null && desti!=null, ExcepcioParametreIncorrecte
    */
   public ArestaDImpl(VertexDImpl<TElem,TEtiqueta> origen, VertexDImpl<TElem,TEtiqueta> desti) {
      super(origen, desti);
   }

   /** Accessor de lectura del vrtex origen de la relaci dirigida.
    * @return  vrtex origen */
   public Vertex<TElem> getVertexOrigen() { return  vertexA; }

   /** Accessor de lectura del vrtex dest de la relaci dirigida.
    * @return  vrtex dest */
   public Vertex<TElem> getVertexDesti() { return  vertexB; }


}