
package uoc.ei.tads.grafs;

import uoc.ei.tads.Posicio;

/**
 * Classe que implementa una aresta.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 * @param <TEtiqueta>
 * @param <TElem>
 */

abstract class ArestaImpl<TEtiqueta,TElem> implements Aresta<TEtiqueta,TElem> {

	protected VertexImpl<TElem,TEtiqueta> vertexA,vertexB;


	/** Etiqueta de l'aresta. */
	protected TEtiqueta etiqueta;


	/** Posicio de l'aresta a la llista d'arestes de GrafAbstracte.
	 * Implementa el patr Locator, i d'aquesta manera aconseguim
	 * poder esborrar l'aresta de la llista en un temps constant
	 * (ja que no cal buscar-la).
	 */
	protected Posicio<Aresta<TEtiqueta,TElem>> posicioLlistaArestesGraf;


	protected Posicio<Aresta<TEtiqueta,TElem>> posicioLlistaDeVertexA,posicioLlistaDeVertexB;


	public ArestaImpl(VertexImpl<TElem,TEtiqueta> vertex1, VertexImpl<TElem,TEtiqueta> vertex2) {
		this.vertexA = vertex1;
		this.vertexB = vertex2;
	}


	/** Retorna l'etiqueta associada a l'aresta. */
	public TEtiqueta getEtiqueta() { return etiqueta; }


	/** Modifica el valor de l'etiqueta associada al node. */
	public void setEtiqueta(TEtiqueta etiqueta) { this.etiqueta = etiqueta; }


	public void setPosicioLlistaGraf(Posicio<Aresta<TEtiqueta,TElem>> posicio) { posicioLlistaArestesGraf=posicio; }
	public void setPosicioLlistaVertexA(Posicio<Aresta<TEtiqueta,TElem>> posicio) { posicioLlistaDeVertexA=posicio; }
	public void setPosicioLlistaVertexB(Posicio<Aresta<TEtiqueta,TElem>> posicio) { posicioLlistaDeVertexB=posicio; }


	/** Consulta si l'aresta incideix en un vrtex.
	 */
	public boolean incideixEnVertex(Vertex<TElem> vertex) {
		return ( this.vertexA.equals(vertex) || this.vertexB.equals(vertex) );
	}


	/** Consulta si l'aresta incideix en tots dos vrtexs a la vegada, sense importar l'ordre.
	 * @param vertex1
	 * @param vertex2
	 * @return
	 */
   public boolean incideixEnVertexs(Vertex<TElem> vertex1,Vertex<TElem> vertex2) {
      return incideixEnVertex(vertex1) && incideixEnVertex(vertex2);
   }


   /** El.limina l'aresta del graf.
    * @param graf Graf al qual pertany l'aresta.
    */
   public void eliminarDelGraf(GrafAbstracte<TElem,TEtiqueta> graf) {
   	  graf.llistaArestes.esborrar(posicioLlistaArestesGraf);
   	  vertexA.eliminarAresta(this);
   	  vertexB.eliminarAresta(this);
   }


   /** Afegeix l'aresta al graf. Els vrtexs associats a l'aresta han de pertnyer al graf.
    * @param graf Graf al qual s'afegeix l'aresta.
    */
   public void afegirAlGraf(GrafAbstracte<TElem,TEtiqueta> graf) {
   		posicioLlistaArestesGraf=graf.llistaArestes.afegirAlFinal(this);
   		vertexA.afegirAresta(this);
   		vertexB.afegirAresta(this);
   }


   /** donat un vrtex pertanyent a l'aresta, aquest mtode retorna l'altre vrtex
    * pertanyent a l'aresta.
    * @param vertexA
    * @return
    */
   public Vertex<TElem> extremAlternatiu(Vertex<TElem> vertex) {
   	  return (vertex==this.vertexA) ? vertexB : vertexA;
   }


   /** Redefinici d'aquest mtode de object.
    */
   public boolean equals(Object object) {
   	  if (getClass()!=object.getClass())
   	  	return false;
   	  ArestaImpl<TEtiqueta,TElem> aresta=(ArestaImpl<TEtiqueta,TElem>)object;
   	  return vertexA.equals(aresta.vertexA) &&
   	  		 vertexB.equals(aresta.vertexB);
   }


   /** Redefinici d'aquest mtode de object.
    */
   public String toString() {
   	  return "("+vertexA+":"+etiqueta+":"+vertexB+")";
   }

}
