package uoc.ei.tads.grafs;

import uoc.ei.tads.Iterador;

/**
 * Grafo.java
 * Grafo (dirigido o no).
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public interface Graf<TElem,TEtiqueta> {

  /**
   * Obtiene un iterador a los vértices del grafo.
   * @pre true
   * @post @return.size()>=1 (un grafo ha de tener al menos un vértice, según la definición matemática).
   */
  public Iterador<Vertex<TElem>> vertexs();


  /** Retorna el nombre de vrtexs del graf.
   * @return n
   */
  public int numVertexs();

   /**
   * Obtiene un iterador a las aristas del grafo.
   * @pre true
   * @post true
   */
  public Iterador<Aresta<TEtiqueta,TElem>> arestes();

   /**
   * Crea un nuevo vértice con un valor asociado, y lo incluye en el grafo.
   * @pre valor != null
   * @post vertices().size()=@old.vertices().size()+1
   * @post @return.obtenerValor().equals(valor);
   */
  public Vertex<TElem> crearVertex(TElem valor);

  /**
   * Elimina el vértice del grafo (si estaba en él), incluyendo todas las aristas en las que participa.
   * @pre n!=null
   * @post @old.vertices().contains(n)=>vertices().size()=@old.vertices().size()-1
   */
  public void eliminarVertex(Vertex<TElem> vertex);


  /** Retorna el vertex corresponent a un element determinat.
   *
   * @param elem
   * @return v
   */
  public Vertex<TElem> consultarVertex(TElem elem);


  /**
   * Elimina la arista del grafo (si estaba en él)
   * @pre a !=null
   * @post @old.aristas().contains(a)=>aristas().size()=@old.aristas().size()-1
   */
  public void eliminarAresta(Aresta<TEtiqueta,TElem> aresta);


  /**
   * Obtiene un iterador a los vértices adyacentes al vértice v.
   * @pre v!=null
   * @post true
   */
  public Iterador<Vertex<TElem>> adjacents(Vertex<TElem> vertex);

}
