
package uoc.ei.tads.grafs;

import java.io.Serializable;

import uoc.ei.tads.*;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
abstract class GrafAbstracte<TElem,TEtiqueta> implements Graf<TElem,TEtiqueta>, Serializable {


	//-----------------------------------------------------------------------
	// atributs
	//-----------------------------------------------------------------------

	LlistaDoblementEncadenada<Aresta<TEtiqueta,TElem>> llistaArestes;


	Diccionari<TElem,Vertex<TElem>> diccionariVertexs;


	//-----------------------------------------------------------------------
	// metodes
	//-----------------------------------------------------------------------

	public GrafAbstracte() {
		diccionariVertexs=crearDiccionariVertexs();
		llistaArestes=new LlistaDoblementEncadenada<Aresta<TEtiqueta,TElem>>();
	}


	protected Diccionari<TElem,Vertex<TElem>> crearDiccionariVertexs() {
		return new DiccionariLlistaImpl<TElem,Vertex<TElem>>();
	}


	public Iterador<Vertex<TElem>> vertexs() {
		return diccionariVertexs.elements();
	}


	public int numVertexs() {
		return diccionariVertexs.nombreElems();
	}


	public Iterador<Aresta<TEtiqueta, TElem>> arestes() {
		return llistaArestes.elements();
	}


	public Vertex<TElem> crearVertex(TElem valor) {
		Vertex<TElem> vertex=nouVertex(valor);
		diccionariVertexs.afegir(valor,vertex);
		return vertex;
	}


	public void eliminarVertex(Vertex<TElem> vertex) {
		VertexImpl<TElem,TEtiqueta> v=(VertexImpl<TElem,TEtiqueta>)vertex;
		Iterador<Aresta<TEtiqueta,TElem>> arestes=v.arestes();
		Llista<Aresta<TEtiqueta,TElem>> arestesAEsborrar=new LlistaEncadenada<Aresta<TEtiqueta,TElem>>();
		while (arestes.hiHaSeguent()) {
			Aresta<TEtiqueta,TElem> aresta=arestes.seguent();
			arestesAEsborrar.afegirAlFinal(aresta);
		}
		while (!arestesAEsborrar.estaBuit()) {
			Aresta<TEtiqueta,TElem> aresta=arestesAEsborrar.esborrarPrimer();
			eliminarAresta(aresta);
		}
		diccionariVertexs.esborrar(vertex.getValor());
	}


	public void eliminarAresta(Aresta<TEtiqueta, TElem> aresta) {
		((ArestaImpl<TEtiqueta,TElem>)aresta).eliminarDelGraf(this);
	}


	public Iterador<Vertex<TElem>> adjacents(Vertex<TElem> vertex) {
		return new IteradorAdjacents<TElem,TEtiqueta>((VertexImpl<TElem,TEtiqueta>)vertex);
	}


	public Vertex<TElem> consultarVertex(TElem elem) {
		return diccionariVertexs.consultar(elem);
	}


	protected abstract VertexImpl<TElem,TEtiqueta> nouVertex(TElem valor);


	public String toString() {
		return "[Graf\n  "+llistaArestes+"\n"+diccionariVertexs+"]";
	}


	//-----------------------------------------------------------------------
	// classes internes: iteradors i recorreguts
	//-----------------------------------------------------------------------

	/** Classe que proporciona un iterador sobre els vertexs adjacents a un vertex donat.
    *
    * @author  Jordi lvarez Canal
    * @author  Esteve Marin Gallis.
    *          Estructura de la Informaci,
    *          Universitat Oberta de Catalunya (UOC)
    * @version  2.0.0
    * @since  1.5
	 *
	 * @param <TElem>
	 * @param <TEtiqueta>
	 */

	static class IteradorAdjacents<TElem,TEtiqueta> implements Iterador<Vertex<TElem>> {

		Iterador<Aresta<TEtiqueta,TElem>> arestes;
		Vertex<TElem> vertex;

		public IteradorAdjacents(VertexImpl<TElem,TEtiqueta> vertex) {
			this.vertex=vertex;
			arestes=vertex.arestes();
		}


		public Vertex<TElem> seguent() {
			ArestaImpl<TEtiqueta,TElem> aresta=(ArestaImpl<TEtiqueta,TElem>)arestes.seguent();
			return aresta.extremAlternatiu(vertex);
		}


		public boolean hiHaSeguent() {
			return arestes.hiHaSeguent();
		}


	}


}
