package uoc.ei.tads.grafs;

import uoc.ei.tads.Iterador;

/**
 * GrafoDirigido.java
 * Grafo dirigido.
 *
 * @author  Jordi Àlvarez Canal
 * @author  Esteve Mariné Gallisà.
 *          Estructura de la Informació,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public interface GrafDirigit<TElem,TEtiqueta> extends Graf<TElem,TEtiqueta> {

 /**
   * Crea e incluye una nueva arista en el grafo.
   * @pre n!=null && m!=null
   * @post (@return.obtenerVertice1()==n &&  @return.obtenerVertice2()==m) ||
           (@return.obtenerVertice1()==m &&  @return.obtenerVertice2()==n)
   */
  public ArestaDirigida<TEtiqueta,TElem> crearAresta(Vertex<TElem> n, Vertex<TElem> m);

  /**
   * Obtiene la arista (si existe) entre los vértices n y m.
   * Devuelve null si no hay una arista entre ellos.
   * @pre n !=null && m!=null
   * @post (@return.vertices().contains(m) &&  @return.vertices().contains(n))||@return==null
   */
  public ArestaDirigida<TEtiqueta,TElem> obtenirAresta(Vertex<TElem> n, Vertex<TElem> m);

	/**
   * Obtiene un iterador a las aristas con origen en v.
   * @pre n!=null && m!=null
   * @post forall @return.value()=> obtenerVerticeOrigen()==v
   */
   public Iterador<Aresta<TEtiqueta,TElem>> arestesAmbOrigenA(Vertex<TElem> v);

   /**
   * Obtiene un iterador a las aristas con destino en v.
   * @pre n!=null && m!=null
   * @post forall @return.value()=> obtenerVerticeDestino()==v
   */
   public Iterador<Aresta<TEtiqueta,TElem>> arestesAmbDestiA(Vertex<TElem> v);
}
