
package uoc.ei.tads.grafs;

import uoc.ei.tads.*;

/** Implementaci d'un graf dirigit mitjanant llistes d'adjacncia.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class GrafDirigitImpl<TElem,TEtiqueta> extends GrafAbstracte<TElem,TEtiqueta>
											implements GrafDirigit<TElem,TEtiqueta> {


   /** Atribut que en Java 1.5 determina la compatibilitat entre objectes
    * serialitzables de la mateixa classe. L'deitificador es calcula
    * mitjanant un mtode de la classe Utilitats. */
   private static final long serialVersionUID = Utilitats.getSerialVersionUID();


	public GrafDirigitImpl() {
		super();
	}


	protected VertexImpl<TElem, TEtiqueta> nouVertex(TElem valor) {
		return new VertexDImpl<TElem,TEtiqueta>(valor);
	}


	public ArestaDirigida<TEtiqueta, TElem> crearAresta(Vertex<TElem> origen,
			Vertex<TElem> desti) {
		VertexDImpl<TElem,TEtiqueta> o=(VertexDImpl<TElem,TEtiqueta>)origen;
		VertexDImpl<TElem,TEtiqueta> d=(VertexDImpl<TElem,TEtiqueta>)desti;
		ArestaDImpl<TEtiqueta,TElem> aresta=new ArestaDImpl<TEtiqueta,TElem>(o,d);
		aresta.afegirAlGraf(this);
		return aresta;
	}


	public ArestaDirigida<TEtiqueta, TElem> obtenirAresta(Vertex<TElem> origen,
			Vertex<TElem> desti) {
		ArestaDirigida<TEtiqueta, TElem> resultat=null;
		Iterador<Aresta<TEtiqueta, TElem>> arestes=arestesAmbOrigenA(origen);
		while (arestes.hiHaSeguent() && resultat==null) {
			ArestaDirigida<TEtiqueta, TElem> aresta=(ArestaDirigida<TEtiqueta, TElem>)arestes.seguent();
			if (aresta.getVertexDesti()==desti)
				resultat=aresta;
		}
		return resultat;
	}


	public Iterador<Aresta<TEtiqueta, TElem>> arestesAmbOrigenA(Vertex<TElem> vertex) {
		VertexDImpl<TElem,TEtiqueta> v=(VertexDImpl<TElem,TEtiqueta>)vertex;
		return v.arestesAmbOrigenAlVertex.elements();
	}


	public Iterador<Aresta<TEtiqueta, TElem>> arestesAmbDestiA(Vertex<TElem> vertex) {
		VertexDImpl<TElem,TEtiqueta> v=(VertexDImpl<TElem,TEtiqueta>)vertex;
		return (Iterador<Aresta<TEtiqueta, TElem>>)v.arestesAmbDestiAlVertex.elements();
	}

}
