package uoc.ei.tads.grafs;

import uoc.ei.tads.Iterador;
import uoc.ei.tads.IteradorMultiple;
import uoc.ei.tads.LlistaDoblementEncadenada;
import uoc.ei.tads.Posicio;
import uoc.ei.tads.Utilitats;

/** Classe que implementa un vrtex per al cas dels Grafs Dirigits.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */

class VertexDImpl<TElem,TEtiqueta> extends VertexImpl<TElem,TEtiqueta> {

   /** Atribut que en Java 1.5 determina la compatibilitat entre objectes
    * serialitzables de la mateixa classe. L'deitificador es calcula
    * mitjanant un mtode de la classe Utilitats. */
   private static final long serialVersionUID = Utilitats.getSerialVersionUID();


   protected LlistaDoblementEncadenada<Aresta<TEtiqueta,TElem>> arestesAmbOrigenAlVertex;
   protected LlistaDoblementEncadenada<Aresta<TEtiqueta,TElem>> arestesAmbDestiAlVertex;


   public VertexDImpl(TElem element) {
   	  super(element);
   	  arestesAmbOrigenAlVertex=new LlistaDoblementEncadenada<Aresta<TEtiqueta,TElem>>();
   	  arestesAmbDestiAlVertex=new LlistaDoblementEncadenada<Aresta<TEtiqueta,TElem>>();
   }


   public void afegirAresta(Aresta<TEtiqueta,TElem> aresta) {
	   	ArestaImpl<TEtiqueta,TElem> arestaI=(ArestaImpl<TEtiqueta,TElem>)aresta;
   		if (arestaI.vertexA==this)
   			arestaI.setPosicioLlistaVertexA(arestesAmbOrigenAlVertex.afegirAlFinal((Aresta<TEtiqueta,TElem>)aresta));
   		if (arestaI.vertexB==this)
   			arestaI.setPosicioLlistaVertexB(arestesAmbDestiAlVertex.afegirAlFinal((Aresta<TEtiqueta,TElem>)aresta));
   }


   public void eliminarAresta(ArestaImpl<TEtiqueta,TElem> aresta) {
   	  Posicio<Aresta<TEtiqueta,TElem>> posicio;
   	  if (aresta.vertexA==this) {
   	  	  posicio=aresta.posicioLlistaDeVertexA;
	   	  arestesAmbOrigenAlVertex.esborrar(posicio);
   	  }
   	  else {
   	  	  posicio=aresta.posicioLlistaDeVertexB;
	   	  arestesAmbDestiAlVertex.esborrar(posicio);
   	  }
   }


   public Iterador<Aresta<TEtiqueta,TElem>> arestes() {
   	  IteradorMultiple<Aresta<TEtiqueta,TElem>> arestes=new IteradorMultiple<Aresta<TEtiqueta,TElem>>();
   	  arestes.afegirIterador(arestesAmbOrigenAlVertex.elements());
   	  arestes.afegirIterador(arestesAmbDestiAlVertex.elements());
   	  return arestes;
   }

   /*
   public String toString() {
   		String str=super.toString()+"\n";
   		str+="  arestes-origen: "+arestesAmbOrigenAlVertex+"\n";
   		str+="  arestes-desti: "+arestesAmbDestiAlVertex+"\n";
   		return str;
   }*/


}