
package uoc.ei.tads.grafs;

import uoc.ei.tads.*;

/** Classe que implementa un vrtex.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */

abstract class VertexImpl<TElem,TEtiqueta> implements Vertex<TElem>
{

	/** Valor del node. */
   private TElem element;


   /** Construeix un vertex a partir del seu valor.
    * @param element
    */
   public VertexImpl(TElem element) {
      this.element = element;
   }


   /** Retorna el valor del node.
    */
   public TElem getValor() { return  element; }


   /** Retorna el conjunt d'arestes del graf en les que el node participa.
    * @return Conjunt d'arestes en les que participa el node.
    */
   public abstract Iterador<Aresta<TEtiqueta,TElem>> arestes();


   /** Elimina la referncia a una aresta.
    * @param aresta Aresta de la qual volem eliminar la referncia.
    */
   public abstract void eliminarAresta(ArestaImpl<TEtiqueta,TElem> aresta);


   /** Afegeix una referncia a l'aresta com a aresta que connecta el vrtex
    * amb un altre vrtex.
    * @param aresta L'aresta afegida.
    * @return
    */
   public abstract void afegirAresta(Aresta<TEtiqueta,TElem> aresta);


   /** Implementaci del mtode heretat d'Object.
    */
   public int hashCode() { return  element.hashCode(); }


   /** Implementaci del mtode heretat d'Object.
    */
   public boolean equals(Object vertex) {
   	  if (getClass()!=vertex.getClass()) return false;
      return element.equals( ((VertexImpl<TElem,TEtiqueta>)vertex).getValor() );
   }


   /** Implementaci del mtode heretat d'Object.
    */
   public String toString() { return "(Vertex:"+element+")"; }

}
