package uoc.ei.tads.tests;

import uoc.ei.tads.Arbre;
import uoc.ei.tads.ArbreAbstracte;
import uoc.ei.tads.Posicio;
import uoc.ei.tads.Recorregut;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class TestArbre extends TestContenidor {


	protected Posicio<Integer> cercaPosicio(Arbre<Integer> arbre,int elem) {
		Posicio<Integer> posicio=null;
		Recorregut<Integer> r=arbre.posicions();
		while (r.hiHaSeguent() && posicio==null) {
			Posicio<Integer> p=r.seguent();
			if (p.getElem()==elem)
				posicio=p;
		}
		return posicio;
	}


	/** Aquest mtode crea un arbre no complet. Cada node t com a mxim dos fills.
	 * Durant el procs de creaci es fan diverses comprovacions sobre els nodes que
	 * es van afegint a l'arbre.
	 * L'arbre creat s:
	 *                                       7
	 *                                  4          2
	 *                                    1      8   15
	 *                                   6           9
	 *                                              5
	 *
	 * Si el contenidor creat s un arbre binari, l'arbre es crea com a arbre binari. Si no,
	 * es fa com a arbre general.
	 * @return
	 */

	protected Arbre<Integer> construirArbre742() {
		ArbreAbstracte<Integer> arbre=(ArbreAbstracte<Integer>)crearContenidor();
		assertTrue(arbre.estaBuit());
		assertTrue(arbre.nombreElems()==0);
		Posicio<Integer> pos7=arbre.afegir(null,7);
		assertTrue(arbre.arrel()==pos7);
		assertTrue(arbre.esFulla(pos7));
		assertTrue(!arbre.estaBuit());
		assertEquals(1,arbre.nombreElems());
		assertEquals(1,arbre.nombreElems(pos7));
		Posicio<Integer> pos4=afegirFillEsquerre(arbre,pos7,4);
		assertTrue(arbre.esFulla(pos4));
		assertTrue(!arbre.esFulla(pos7));
		assertEquals(2,arbre.nombreElems());
		assertEquals(2,arbre.nombreElems(pos7));
		Posicio<Integer> pos2=afegirFillDret(arbre,pos7,2);
		assertTrue(!arbre.esFulla(pos7));
		assertTrue(arbre.esFulla(pos2));
		assertEquals(3,arbre.nombreElems());
		assertEquals(3,arbre.nombreElems(pos7));
		Posicio<Integer> pos1=afegirFillDret(arbre,pos4,1);
		assertTrue(arbre.esFulla(pos1));
		assertTrue(!arbre.esFulla(pos4));
		assertEquals(4,arbre.nombreElems());
		assertEquals(2,arbre.nombreElems(pos4));
		Posicio<Integer> pos6=afegirFillEsquerre(arbre,pos1,6);
		assertTrue(!arbre.esFulla(pos1));
		assertTrue(arbre.esFulla(pos6));
		assertEquals(5,arbre.nombreElems());
		assertEquals(2,arbre.nombreElems(pos1));
		Posicio<Integer> pos8=afegirFillEsquerre(arbre,pos2,8);
		assertTrue(!arbre.esFulla(pos2));
		assertTrue(arbre.esFulla(pos8));
		assertEquals(6,arbre.nombreElems());
		assertEquals(2,arbre.nombreElems(pos2));
		Posicio<Integer> pos15=afegirFillDret(arbre,pos2,15);
		assertTrue(!arbre.esFulla(pos2));
		assertTrue(arbre.esFulla(pos15));
		assertEquals(7,arbre.nombreElems());
		assertEquals(3,arbre.nombreElems(pos2));
		Posicio<Integer> pos9=afegirFillEsquerre(arbre,pos15,9);
		assertTrue(arbre.esFulla(pos9));
		assertTrue(!arbre.esFulla(pos15));
		assertEquals(8,arbre.nombreElems());
		Posicio<Integer> pos5=afegirFillEsquerre(arbre,pos9,5);
		assertTrue(!arbre.esFulla(pos9));
		assertTrue(arbre.esFulla(pos5));
		assertTrue(arbre.arrel()==pos7);
		assertEquals(9,arbre.nombreElems());
		assertEquals(9,arbre.nombreElems(pos7));
		assertEquals(5,arbre.nombreElems(pos2));
		return arbre;
	}


	protected Posicio<Integer> afegirFillEsquerre(Arbre<Integer> arbre,Posicio<Integer> pare,int element) {
		return arbre.afegir(pare, element);
	}


	protected Posicio<Integer> afegirFillDret(ArbreAbstracte<Integer> arbre,Posicio<Integer> pare,int element) {
		return arbre.afegir(pare, element);
	}


	public void testCrear742() {
		construirArbre742();
	}

	public void testRecorregutPerNivells742() {
		Arbre<Integer> arbre=construirArbre742();
		Recorregut<Integer> r=arbre.recorregutPerNivells();
		int[] elems={7,4,2,1,8,15,6,9,5};
		checkRecorregut(r,elems);
	}


	public void testRecorregutPreordre742() {
		Arbre<Integer> arbre=construirArbre742();
		Recorregut<Integer> r=arbre.recorregutPreordre();
		int[] elems={7,4,1,6,2,8,15,9,5};
		checkRecorregut(r,elems);
	}


	public void testRecorregutPostordre742() {
		Arbre<Integer> arbre=construirArbre742();
		Recorregut<Integer> r=arbre.recorregutPostordre();
		int[] elems={6,1,4,8,5,9,15,2,7};
		checkRecorregut(r,elems);
	}


	public void testEsborrar742() {
		Arbre<Integer> arbre=construirArbre742();
		Posicio<Integer> pos2=cercaPosicio(arbre,2);
		Posicio<Integer> pos15=cercaPosicio(arbre,15);
		arbre.esborrar(pos2,pos15);
		assertEquals(6,arbre.nombreElems());
		assertEquals(1,comptarElems(arbre.fills(pos2)));
		Posicio<Integer> pos1=cercaPosicio(arbre,1);
		Posicio<Integer> pos6=cercaPosicio(arbre,6);
		arbre.esborrar(pos1,pos6);
		assertEquals(5,arbre.nombreElems());
		assertEquals(0,comptarElems(arbre.fills(pos1)));
	}


	public void testIntercanviar742() {
		Arbre<Integer> arbre=construirArbre742();
		Posicio<Integer> pos4=cercaPosicio(arbre,4);
		Posicio<Integer> pos15=cercaPosicio(arbre,15);
		Posicio<Integer> pos8=cercaPosicio(arbre,8);
		arbre.intercanviar(pos8, pos15);
		Recorregut<Integer> r=arbre.recorregutPerNivells();
		int[] elems={7,4,2,1,15,8,6,9,5};
		checkRecorregut(r,elems);
		arbre.intercanviar(pos15, pos4);
		arbre.intercanviar(pos4, pos8);
		arbre.intercanviar(pos4, pos15);
		r=arbre.recorregutPerNivells();
		int[] elems2={7,4,2,1,8,15,6,9,5};
		checkRecorregut(r,elems2);
	}


	public void testReemplacar742() {
		Arbre<Integer> arbre=construirArbre742();
		Posicio<Integer> pos4=cercaPosicio(arbre,4);
		Posicio<Integer> pos15=cercaPosicio(arbre,15);
		Posicio<Integer> pos8=cercaPosicio(arbre,8);
		arbre.reemplacar(pos8,15);
		arbre.reemplacar(pos15,8);
		Recorregut<Integer> r=arbre.recorregutPerNivells();
		int[] elems={7,4,2,1,15,8,6,9,5};
		checkRecorregut(r,elems);
		arbre.reemplacar(pos4, 16);
		arbre.reemplacar(pos8,17);
		arbre.reemplacar(pos15,18);
		r=arbre.recorregutPerNivells();
		int[] elems2={7,16,2,1,17,18,6,9,5};
		checkRecorregut(r,elems2);
	}


}
