package uoc.ei.tads.tests;

import java.util.ArrayList;

import uoc.ei.tads.Conjunt;
import uoc.ei.tads.Contenidor;
import uoc.ei.tads.ExcepcioPosicioInvalida;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public abstract class TestConjunt extends TestContenidor {


	protected void afegir(Contenidor<Integer> c,int elem) {
		((Conjunt<Integer>)c).afegir(elem);
	}


	protected boolean esborrar(Contenidor<Integer> c,int elem) {
		return ((Conjunt<Integer>)c).esborrar(elem)!=null;
	}


	protected int esborrarElement(Contenidor<Integer> contenidor,ArrayList<Integer> elements) {
		int element=super.esborrarElement(contenidor,elements);
		while (elements.remove(new Integer(element))) {}
		return element;
	}


	public void testBuit() {
		Conjunt<Integer> conjunt=(Conjunt<Integer>)crearContenidor();
		assertTrue(conjunt.estaBuit());
		assertFalse(conjunt.elements().hiHaSeguent());
		try {
			conjunt.elements().seguent();
			fail("el diccionari s buit i caldria que s'hagus generat ExcepcioPosicioInvalida");
		} catch (ExcepcioPosicioInvalida e) {}
		assertFalse(conjunt.hiEs(5));
		assertNull(conjunt.esborrar(5));
	}


	public void testOrdre() {
		Conjunt<Integer> conjunt=(Conjunt<Integer>)crearContenidor();
		afegir(conjunt,7);
		afegir(conjunt,3);
		afegir(conjunt,5);
		afegir(conjunt,1);
		afegir(conjunt,9);
		afegir(conjunt,8);
		afegir(conjunt,6);
		afegir(conjunt,4);
		afegir(conjunt,2);
		afegir(conjunt,0);
		estaOrdenat(conjunt.elements());
	}


	public void testOrdreRepetits() {
		Conjunt<Integer> conjunt=(Conjunt<Integer>)crearContenidor();
		afegir(conjunt,7);
		afegir(conjunt,3);
		afegir(conjunt,3);
		afegir(conjunt,1);
		afegir(conjunt,7);
		afegir(conjunt,7);
		afegir(conjunt,7);
		afegir(conjunt,3);
		afegir(conjunt,5);
		afegir(conjunt,1);
		estaOrdenat(conjunt.elements());
	}


	public void testOmplirBuidar() {
		Conjunt<Integer> conjunt=(Conjunt<Integer>)crearContenidor();
		afegir(conjunt,7);
		assertFalse(conjunt.estaBuit());
		afegir(conjunt,3);
		afegir(conjunt,5);
		afegir(conjunt,1);
		afegir(conjunt,9);
		assertEquals(1,(int)conjunt.esborrar(1));
		assertEquals(null,conjunt.esborrar(1));
		assertEquals(3,(int)conjunt.esborrar(3));
		assertEquals(null,conjunt.esborrar(3));
		assertEquals(5,(int)conjunt.esborrar(5));
		assertEquals(null,conjunt.esborrar(5));
		assertEquals(9,(int)conjunt.esborrar(9));
		assertEquals(null,conjunt.esborrar(9));
		assertEquals(7,(int)conjunt.esborrar(7));
		assertEquals(null,conjunt.esborrar(9));
		assertEquals(0,conjunt.nombreElems());
		assertTrue(conjunt.estaBuit());
	}


	protected void testAleatori(int numElems,int rangMaxim) {
		Conjunt<Integer> conjunt=(Conjunt<Integer>)crearContenidor();
		for (int i=0;i<NUM_TESTS_ALEATORIS;i++) {
			ArrayList<Integer> elements=popularAleatoriament(conjunt,numElems,rangMaxim);
			estaOrdenat(conjunt.elements());
			for(int j=0;j<numElems && !conjunt.estaBuit();j++) {
				esborrarElement(conjunt,elements);
				estaOrdenat(conjunt.elements());
			}
			assertTrue(conjunt.estaBuit());
		}
	}


	public void testAleatoriRangGran() {
		testAleatori(NUM_ELEMS_TEST_ALEATORI,NUM_ELEMS_TEST_ALEATORI*10);
	}


	public void testAleatoriRangPetit() {
		testAleatori(NUM_ELEMS_TEST_ALEATORI,NUM_ELEMS_TEST_ALEATORI/10);
	}


}
