package uoc.ei.tads.tests;

import uoc.ei.tads.Contenidor;
import uoc.ei.tads.CuaAmbPrioritat;
import uoc.ei.tads.ExcepcioContenidorBuit;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class TestCuaAmbPrioritat extends TestContenidor {


	protected void afegir(Contenidor<Integer> c,int elem) {
		CuaAmbPrioritat<Integer> cua=(CuaAmbPrioritat<Integer>)c;
		cua.encuar(elem);
	}


	protected Contenidor<Integer> crearContenidor() {
		return new CuaAmbPrioritat<Integer>();
	}


	protected Contenidor<Integer> crearContenidor(int numElems) {
		return new CuaAmbPrioritat<Integer>(numElems);
	}


	/** Aquest test comprova els mtodes de la cua amb prioritat per a la
	 * cua buida. Operacions provades: estaBuit, estaPle, desencuar, primer,
	 * nombreElems, elements, posicions.
	 */

	public void testCuaBuida() {
		CuaAmbPrioritat<Integer> cua=(CuaAmbPrioritat<Integer>)crearContenidor();
		assertTrue(cua.estaBuit());
		try {
			cua.desencuar();
		} catch (ExcepcioContenidorBuit e) {}
		try {
			cua.primer();
		} catch (ExcepcioContenidorBuit e) {}
		assertFalse(cua.elements().hiHaSeguent());
		assertFalse(cua.estaPle());
		assertEquals(0,cua.nombreElems());
		assertFalse(cua.elements().hiHaSeguent());
		assertFalse(cua.posicions().hiHaSeguent());
	}


	public void testOrdre() {
		CuaAmbPrioritat<Integer> cua=(CuaAmbPrioritat<Integer>)crearContenidor();
		cua.encuar(7);
		cua.encuar(3);
		cua.encuar(5);
		cua.encuar(1);
		cua.encuar(9);
		cua.encuar(8);
		cua.encuar(6);
		cua.encuar(4);
		cua.encuar(2);
		cua.encuar(0);
		estaOrdenat(cua.elements());
	}


	public void testOrdreRepetits() {
		CuaAmbPrioritat<Integer> cua=(CuaAmbPrioritat<Integer>)crearContenidor();
		cua.encuar(7);
		cua.encuar(3);
		cua.encuar(3);
		cua.encuar(1);
		cua.encuar(7);
		cua.encuar(7);
		cua.encuar(7);
		cua.encuar(3);
		cua.encuar(5);
		cua.encuar(1);
		estaOrdenat(cua.elements());
		assertEquals(1,(int)cua.desencuar());
		assertEquals(1,(int)cua.desencuar());
		assertEquals(3,(int)cua.desencuar());
		assertEquals(3,(int)cua.desencuar());
		assertEquals(3,(int)cua.desencuar());
		assertEquals(5,(int)cua.desencuar());
		assertEquals(7,(int)cua.desencuar());
		assertEquals(7,(int)cua.desencuar());
		assertEquals(7,(int)cua.desencuar());
		assertEquals(7,(int)cua.desencuar());
		assertEquals(0,cua.nombreElems());
		assertTrue(cua.estaBuit());
	}


	public void testOmplirBuidar() {
		CuaAmbPrioritat<Integer> cua=(CuaAmbPrioritat<Integer>)crearContenidor();
		cua.encuar(7);
		cua.encuar(3);
		cua.encuar(5);
		cua.encuar(1);
		cua.encuar(9);
		assertEquals(1,(int)cua.desencuar());
		assertEquals(3,(int)cua.desencuar());
		assertEquals(5,(int)cua.desencuar());
		assertEquals(7,(int)cua.desencuar());
		assertEquals(9,(int)cua.desencuar());
		assertEquals(0,cua.nombreElems());
		assertTrue(cua.estaBuit());
		try {
			cua.desencuar();
			fail("la cua est buida!!!");
		} catch (ExcepcioContenidorBuit e) {}
	}


	protected void testAleatori(int numElems,int rangMaxim) {
		CuaAmbPrioritat<Integer> cua=(CuaAmbPrioritat<Integer>)crearContenidor(numElems);
		for (int i=0;i<NUM_TESTS_ALEATORIS;i++) {
			popularAleatoriament(cua,numElems,rangMaxim);
			estaOrdenat(cua.elements());
			int elemAnt=Integer.MIN_VALUE,elem;
			for(int j=0;j<numElems;j++) {
				elem=cua.desencuar();
				assertTrue(elemAnt<=elem);
				elemAnt=elem;
			}
			assertTrue(cua.estaBuit());
		}
	}


	public void testAleatoriRangGran() {
		testAleatori(NUM_ELEMS_TEST_ALEATORI,NUM_ELEMS_TEST_ALEATORI*10);
	}


	public void testAleatoriRangPetit() {
		testAleatori(NUM_ELEMS_TEST_ALEATORI,NUM_ELEMS_TEST_ALEATORI/10);
	}


}
