package uoc.ei.tads.tests;

import uoc.ei.tads.Contenidor;
import uoc.ei.tads.Cua;
import uoc.ei.tads.CuaVectorImpl;
import uoc.ei.tads.ExcepcioContenidorBuit;
import uoc.ei.tads.ExcepcioContenidorPle;

/** Conjunt de test cases sobre la implementaci amb vector del TAD cua.
 * CuaVectorImpl.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */

public class TestCuaVectorImpl extends TestContenidor {


	protected Contenidor<Integer> crearContenidor() {
		return new CuaVectorImpl<Integer>();
	}


	protected Contenidor<Integer> crearContenidor(int numElems) {
		return new CuaVectorImpl<Integer>(numElems);
	}


	/** Comprova el llenament de l'excepci corresponent quan la cua s
	 * buida i intentem desencuar.
	 * Es comproven les operacions: encuar, desencuar, estaBuit.
	 */

	public void testBuit() {
		Cua<Integer> cua=(CuaVectorImpl<Integer>)crearContenidor();
		cua.encuar(5);
		int elem=cua.desencuar();
		assertEquals(5,elem);
		assertTrue(cua.estaBuit());
		try {
			cua.desencuar();
			fail("Expected ExcepcioContenidorBuit");
		} catch (ExcepcioContenidorBuit e) {}
		cua.encuar(8);
		elem=cua.desencuar();
		assertEquals(8,elem);
		assertTrue(cua.estaBuit());
	}


	/** Comprova el llenament de l'excepci corresponent quan la cua s
	 * plena i intentem encuar.
	 * Es comproven les operacions: encuar, desencuar, estaPle.
	 */

	public void testPle() {
		CuaVectorImpl<Integer> cua=(CuaVectorImpl<Integer>)crearContenidor(MIDA_PLENA);
		cua.encuar(5);
		cua.encuar(7);
		cua.encuar(9);
		assertTrue(cua.estaPle());
		try {
			cua.encuar(12);
			fail("Expected ExcepcioContenidorPle");
		} catch (ExcepcioContenidorPle e) {}
		int elem=cua.desencuar();
		assertEquals(5,elem);
		elem=cua.desencuar();
		assertEquals(7,elem);
		elem=cua.desencuar();
		assertEquals(9,elem);
		assertTrue(cua.estaBuit());
	}


	/** Comprova que els elements s'empilen en una cua i es desempilen
	 * en l'ordre esperat.
	 * Es comproven les operacions: encuar, desencuar.
	 */

	public void testOrdre() {
		Cua<Integer> cua=(CuaVectorImpl<Integer>)crearContenidor();
		cua.encuar(5);
		cua.encuar(7);
		cua.encuar(9);
		int elem=cua.desencuar();
		assertEquals(5,elem);
		elem=cua.desencuar();
		assertEquals(7,elem);
		elem=cua.desencuar();
		assertEquals(9,elem);
	}


	/** Aquest test realitza moltes operacions d'encuar i desencuar
	 * sobre una mateixa cua. Omple i buida la cua un nombre elevat
	 * de cops, comprovant cada cop que l'ordre en el que s'obtenen els
	 * elements s el correcte.
	 * Es comproven les operacions: encuar, desencuar, estaBuit, estaPle, primer.
	 */

	public void testMoltesOperacions() {
		CuaVectorImpl<Integer> cua=(CuaVectorImpl<Integer>)crearContenidor(MIDA_MOLTES_OPS);
		for (int i=0;i<REPETICIONS_MOLTES_OPS;i++) {
			for(int j=0;j<MIDA_MOLTES_OPS;j++) {
				cua.encuar(j);
				int elem=cua.primer();
				assertEquals(0,elem);
			}
			assertTrue(cua.estaPle());
			for(int j=0;j<MIDA_MOLTES_OPS;j++) {
				int primer=cua.primer();
				assertEquals(j,primer);
				int elem=cua.desencuar();
				assertEquals(j,elem);
			}
			assertTrue(cua.estaBuit());
		}
	}


}
