package uoc.ei.tads.tests;

import java.util.ArrayList;

import uoc.ei.tads.Contenidor;
import uoc.ei.tads.Diccionari;
import uoc.ei.tads.ExcepcioPosicioInvalida;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public abstract class TestDiccionari extends TestContenidor {



	protected void afegir(Contenidor<Integer> c,int elem) {
		((Diccionari<Integer,Integer>)c).afegir(elem,calcularValor(elem));
	}


	protected boolean esborrar(Contenidor<Integer> c,int elem) {
		return ((Diccionari<Integer,Integer>)c).esborrar(elem)!=null;
	}


	protected int calcularValor(int clau) { return clau*2; }


	protected int esborrarElement(Contenidor<Integer> contenidor,ArrayList<Integer> elements) {
		int element=super.esborrarElement(contenidor,elements);
		while (elements.remove(new Integer(element))) {}
		return element;
	}


	public void testDiccionariBuit() {
		Diccionari<Integer,Integer> diccionari=(Diccionari<Integer,Integer>)crearContenidor();
		assertTrue(diccionari.estaBuit());
		assertFalse(diccionari.claus().hiHaSeguent());
		assertFalse(diccionari.elements().hiHaSeguent());
		try {
			diccionari.claus().seguent();
			fail("el diccionari s buit i caldria que s'hagus generat ExcepcioPosicioInvalida");
		} catch (ExcepcioPosicioInvalida e) {}
		try {
			diccionari.elements().seguent();
			fail("el diccionari s buit i caldria que s'hagus generat ExcepcioPosicioInvalida");
		} catch (ExcepcioPosicioInvalida e) {}
		assertNull(diccionari.consultar(5));
		assertNull(diccionari.esborrar(5));
	}


	public void testOrdre() {
		Diccionari<Integer,Integer> diccionari=(Diccionari<Integer,Integer>)crearContenidor();
		afegir(diccionari,7);
		afegir(diccionari,3);
		afegir(diccionari,5);
		afegir(diccionari,1);
		afegir(diccionari,9);
		afegir(diccionari,8);
		afegir(diccionari,6);
		afegir(diccionari,4);
		afegir(diccionari,2);
		afegir(diccionari,0);
		estaOrdenat(diccionari.elements());
	}


	public void testOrdreRepetits() {
		Diccionari<Integer,Integer> diccionari=(Diccionari<Integer,Integer>)crearContenidor();
		afegir(diccionari,7);
		afegir(diccionari,3);
		afegir(diccionari,3);
		afegir(diccionari,1);
		afegir(diccionari,7);
		afegir(diccionari,7);
		afegir(diccionari,7);
		afegir(diccionari,3);
		afegir(diccionari,5);
		afegir(diccionari,1);
		estaOrdenat(diccionari.elements());
	}


	public void testOmplirBuidar() {
		Diccionari<Integer,Integer> diccionari=(Diccionari<Integer,Integer>)crearContenidor();
		afegir(diccionari,7);
		assertFalse(diccionari.estaBuit());
		afegir(diccionari,3);
		afegir(diccionari,5);
		afegir(diccionari,1);
		afegir(diccionari,9);
		assertEquals(calcularValor(1),(int)diccionari.esborrar(1));
		assertEquals(null,diccionari.esborrar(1));
		assertEquals(calcularValor(3),(int)diccionari.esborrar(3));
		assertEquals(null,diccionari.esborrar(3));
		assertEquals(calcularValor(5),(int)diccionari.esborrar(5));
		assertEquals(null,diccionari.esborrar(5));
		assertEquals(calcularValor(9),(int)diccionari.esborrar(9));
		assertEquals(null,diccionari.esborrar(9));
		assertEquals(calcularValor(7),(int)diccionari.esborrar(7));
		assertEquals(null,diccionari.esborrar(9));
		assertEquals(0,diccionari.nombreElems());
		assertTrue(diccionari.estaBuit());
	}


	protected void testAleatori(int numElems,int rangMaxim) {
		Diccionari<Integer,Integer> diccionari=(Diccionari<Integer,Integer>)crearContenidor();
		for (int i=0;i<NUM_TESTS_ALEATORIS;i++) {
			ArrayList<Integer> elements=popularAleatoriament(diccionari,numElems,rangMaxim);
			estaOrdenat(diccionari.elements());
			for(int j=0;j<numElems && !diccionari.estaBuit();j++) {
				esborrarElement(diccionari,elements);
				estaOrdenat(diccionari.elements());
			}
			assertTrue(diccionari.estaBuit());
		}
	}


	public void testAleatoriRangGran() {
		testAleatori(NUM_ELEMS_TEST_ALEATORI,NUM_ELEMS_TEST_ALEATORI*10);
	}


	public void testAleatoriRangPetit() {
		testAleatori(NUM_ELEMS_TEST_ALEATORI,NUM_ELEMS_TEST_ALEATORI/10);
	}


}
