package uoc.ei.tads.tests;

import uoc.ei.tads.Iterador;
import uoc.ei.tads.grafs.Aresta;
import uoc.ei.tads.grafs.Graf;
import uoc.ei.tads.grafs.GrafNoDirigit;
import uoc.ei.tads.grafs.Vertex;

/**
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public abstract class TestGraf extends TestContenidor {


	protected abstract Aresta <Integer,Character> crearAresta(Graf<Character,Integer> graf,Vertex<Character> v1,Vertex<Character> v2);
	protected abstract Aresta <Integer,Character> obtenirAresta(Graf<Character,Integer> graf,Vertex<Character> v1,Vertex<Character> v2);
	protected abstract Graf<Character,Integer> crearGraf();


	protected void checkVertexs(Iterador<Vertex<Character>> i,char[] etiquetes) {
		boolean etiquetaTrobada=true;
		boolean[] etiquetesVisitades=new boolean[etiquetes.length];
		while (i.hiHaSeguent() && etiquetaTrobada) {
			char etiqueta=i.seguent().getValor();
			int posicio=0;
			boolean posicioTrobada=false;
			while (posicio<etiquetes.length && !posicioTrobada) {
				posicioTrobada=!etiquetesVisitades[posicio] && etiqueta==etiquetes[posicio];
				if (!posicioTrobada)
					posicio++;
			}
			if (posicioTrobada)
				etiquetesVisitades[posicio]=true;
			else {
				etiquetaTrobada=false;
				fail("label "+etiqueta+" not found in defined set of labels");
			}
		}
		for (int j=0;j<etiquetes.length;j++)
			if (!etiquetesVisitades[j])
				fail("label "+etiquetes[j]+" not present in Iterador of Vertex");
	}


	protected void checkArestes(Iterador<Aresta<Integer,Character>> i,int[] etiquetes) {
		boolean etiquetaTrobada=true;
		boolean[] etiquetesVisitades=new boolean[etiquetes.length];
		while (i.hiHaSeguent() && etiquetaTrobada) {
			int etiqueta=i.seguent().getEtiqueta();
			int posicio=0;
			boolean posicioTrobada=false;
			while (posicio<etiquetes.length && !posicioTrobada) {
				posicioTrobada=!etiquetesVisitades[posicio] && etiqueta==etiquetes[posicio];
				if (!posicioTrobada)
					posicio++;
			}
			if (posicioTrobada)
				etiquetesVisitades[posicio]=true;
			else {
				etiquetaTrobada=false;
				fail("label "+etiqueta+" not found in defined set of labels");
			}
		}
		for (int j=0;j<etiquetes.length;j++)
			if (!etiquetesVisitades[j])
				fail("label "+etiquetes[j]+" not present in Iterador of Aresta");
	}


	/** Aquest mtode crea un graf. Serveix tant per a grafs dirigits com per a grafs
	 * no dirigits.
	 * @return
	 */

	protected Graf<Character,Integer> construirGrafABC() {
		Graf<Character,Integer> graf=crearGraf();
		assertFalse(graf.vertexs().hiHaSeguent());
		assertFalse(graf.arestes().hiHaSeguent());
		Vertex<Character> va=graf.crearVertex('A');
		Vertex<Character> vb=graf.crearVertex('B');
		Vertex<Character> vc=graf.crearVertex('C');
		Vertex<Character> vd=graf.crearVertex('D');
		assertEquals(4,graf.numVertexs());
		assertEquals(4,comptarElems(graf.vertexs()));
		assertFalse(graf.arestes().hiHaSeguent());
		Vertex<Character> ve=graf.crearVertex('E');
		Vertex<Character> vf=graf.crearVertex('F');
		Vertex<Character> vg=graf.crearVertex('G');
		Vertex<Character> vh=graf.crearVertex('H');
		assertEquals(8,graf.numVertexs());
		assertEquals(8,comptarElems(graf.vertexs()));
		assertFalse(graf.arestes().hiHaSeguent());
		Aresta<Integer,Character> arestaAB=crearAresta(graf,vb,va);
		arestaAB.setEtiqueta(2);
		Aresta<Integer,Character> arestaAC=crearAresta(graf,va,vc);
		arestaAC.setEtiqueta(5);
		Aresta<Integer,Character> arestaAD=crearAresta(graf,vd,va);
		arestaAD.setEtiqueta(10);
		assertEquals(3,comptarElems(graf.arestes()));
		Aresta<Integer,Character> arestaAE=crearAresta(graf,va,ve);
		arestaAE.setEtiqueta(5);
		Aresta<Integer,Character> arestaAF=crearAresta(graf,va,vf);
		arestaAF.setEtiqueta(4);
		Aresta<Integer,Character> arestaBG=crearAresta(graf,vg,vb);
		arestaBG.setEtiqueta(1);
		assertEquals(6,comptarElems(graf.arestes()));
		Aresta<Integer,Character> arestaBH=crearAresta(graf,vb,vh);
		arestaBH.setEtiqueta(7);
		Aresta<Integer,Character> arestaCG=crearAresta(graf,vg,vc);
		arestaCG.setEtiqueta(9);
		Aresta<Integer,Character> arestaEH=crearAresta(graf,vh,ve);
		arestaEH.setEtiqueta(18);
		Aresta<Integer,Character> arestaFH=crearAresta(graf,vf,vh);
		arestaFH.setEtiqueta(2);
		assertEquals(10,comptarElems(graf.arestes()));
		assertEquals(va,graf.consultarVertex('A'));
		assertEquals(vb,graf.consultarVertex('B'));
		assertEquals(vc,graf.consultarVertex('C'));
		assertEquals(vd,graf.consultarVertex('D'));
		assertEquals(ve,graf.consultarVertex('E'));
		assertEquals(vf,graf.consultarVertex('F'));
		assertEquals(vg,graf.consultarVertex('G'));
		assertEquals(vh,graf.consultarVertex('H'));
		assertEquals(arestaAB,obtenirAresta(graf, vb, va));
		assertEquals(arestaAC,obtenirAresta(graf, va, vc));
		assertEquals(arestaAD,obtenirAresta(graf, vd, va));
		assertEquals(arestaAE,obtenirAresta(graf, va, ve));
		assertEquals(arestaAF,obtenirAresta(graf, va, vf));
		assertEquals(arestaBG,obtenirAresta(graf, vg, vb));
		assertEquals(arestaBH,obtenirAresta(graf, vb, vh));
		assertEquals(arestaCG,obtenirAresta(graf, vg, vc));
		assertEquals(arestaEH,obtenirAresta(graf, vh, ve));
		assertEquals(arestaFH,obtenirAresta(graf, vf, vh));
		if (graf instanceof GrafNoDirigit) {
			assertEquals(arestaAB,obtenirAresta(graf, vb, va));
			assertEquals(arestaAC,obtenirAresta(graf, vc, va));
			assertEquals(arestaAD,obtenirAresta(graf, va, vd));
			assertEquals(arestaAE,obtenirAresta(graf, ve, va));
			assertEquals(arestaAF,obtenirAresta(graf, vf, va));
			assertEquals(arestaBG,obtenirAresta(graf, vb, vg));
			assertEquals(arestaBH,obtenirAresta(graf, vh, vb));
			assertEquals(arestaCG,obtenirAresta(graf, vc, vg));
			assertEquals(arestaEH,obtenirAresta(graf, ve, vh));
			assertEquals(arestaFH,obtenirAresta(graf, vh, vf));
		}
		return graf;
	}


	public void testCrearGrafABC() {
		construirGrafABC();
	}


	public void testAdjacents() {
		Graf<Character,Integer> graf=construirGrafABC();
		Vertex<Character> va=graf.consultarVertex('A');
		Iterador<Vertex<Character>> i=graf.adjacents(va);
		char[] valorsA={'B', 'C', 'D', 'E', 'F'};
		checkVertexs(i,valorsA);
		Vertex<Character> vd=graf.consultarVertex('D');
		i=graf.adjacents(vd);
		char[] valorsD={'A'};
		checkVertexs(i,valorsD);
		Vertex<Character> ve=graf.consultarVertex('E');
		i=graf.adjacents(ve);
		char[] valorsE={'A', 'H'};
		checkVertexs(i,valorsE);
	}


	public void testVertexs() {
		Graf<Character,Integer> graf=construirGrafABC();
		char[] etiquetes={'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H'};
		checkVertexs(graf.vertexs(),etiquetes);
	}


	public void testArestes() {
		Graf<Character,Integer> graf=construirGrafABC();
		int[] etiquetes={1, 2, 2, 4, 5, 5, 7, 9, 10, 18};
		checkArestes(graf.arestes(),etiquetes);
	}


	public void testObtenirAresta() {
		Graf<Character,Integer> graf=construirGrafABC();
		Vertex<Character> va=graf.consultarVertex('A');
		Vertex<Character> vc=graf.consultarVertex('C');
		Vertex<Character> vg=graf.consultarVertex('G');
		Vertex<Character> vb=graf.consultarVertex('B');
		Aresta<Integer,Character> aresta=obtenirAresta(graf,va,vc);
		assertEquals(5,(int)aresta.getEtiqueta());
		aresta=obtenirAresta(graf,vg,vc);
		assertEquals(9,(int)aresta.getEtiqueta());
		aresta=obtenirAresta(graf,vg,vb);
		assertEquals(1,(int)aresta.getEtiqueta());
		aresta=obtenirAresta(graf,vb,va);
		assertEquals(2,(int)aresta.getEtiqueta());
	}


	public void testEliminarVertex() {
		Graf<Character,Integer> graf=construirGrafABC();
		graf.eliminarVertex(graf.consultarVertex('B'));
		assertEquals(7,graf.numVertexs());
		char[] etiquetes1={'A', 'C', 'D', 'E', 'F', 'G', 'H'};
		checkVertexs(graf.vertexs(),etiquetes1);

		graf.eliminarVertex(graf.consultarVertex('D'));
		assertEquals(6,graf.numVertexs());
		char[] etiquetes2={'A', 'C', 'E', 'F', 'G', 'H'};
		checkVertexs(graf.vertexs(),etiquetes2);

		graf.eliminarVertex(graf.consultarVertex('F'));
		assertEquals(5,graf.numVertexs());
		char[] etiquetes3={'A', 'C', 'E', 'G', 'H'};
		checkVertexs(graf.vertexs(),etiquetes3);

		Vertex<Character> va=graf.consultarVertex('A');
		Iterador<Vertex<Character>> i=graf.adjacents(va);
		char[] valorsA={'C', 'E'};
		checkVertexs(i,valorsA);

		Vertex<Character> vd=graf.consultarVertex('D');
		assertNull(vd);

		Vertex<Character> ve=graf.consultarVertex('E');
		i=graf.adjacents(ve);
		char[] valorsE={'A', 'H'};
		checkVertexs(i,valorsE);
	}


	public void testEliminarAresta() {
		Graf<Character,Integer> graf=construirGrafABC();
		Vertex<Character> va=graf.consultarVertex('A');
		Vertex<Character> vb=graf.consultarVertex('B');
		Vertex<Character> vc=graf.consultarVertex('C');
		Vertex<Character> vd=graf.consultarVertex('D');
		Vertex<Character> vf=graf.consultarVertex('F');
		Vertex<Character> vg=graf.consultarVertex('G');
		Vertex<Character> vh=graf.consultarVertex('H');

		graf.eliminarAresta(obtenirAresta(graf, vb, vh));
		graf.eliminarAresta(obtenirAresta(graf, va, vf));
		graf.eliminarAresta(obtenirAresta(graf, vd, va));
		graf.eliminarAresta(obtenirAresta(graf, vg, vc));

		assertNull(obtenirAresta(graf, vb, vh));
		assertNull(obtenirAresta(graf, va, vf));
		assertNull(obtenirAresta(graf, vd, va));
		assertNull(obtenirAresta(graf, vg, vc));

		assertNull(obtenirAresta(graf, vh, vb));
		assertNull(obtenirAresta(graf, vf, va));
		assertNull(obtenirAresta(graf, va, vd));
		assertNull(obtenirAresta(graf, vc, vg));

		int[] etiquetes={1, 2, 2, 5, 5, 18};
		checkArestes(graf.arestes(),etiquetes);

		char[] valorsA={'B', 'C', 'E'};
		checkVertexs(graf.adjacents(va),valorsA);
		char[] valorsB={'G', 'A'};
		checkVertexs(graf.adjacents(vb),valorsB);
		char[] valorsC={'A'};
		checkVertexs(graf.adjacents(vc),valorsC);
		char[] valorsD={};
		checkVertexs(graf.adjacents(vd),valorsD);
		char[] valorsF={'H'};
		checkVertexs(graf.adjacents(vf),valorsF);
		char[] valorsG={'B'};
		checkVertexs(graf.adjacents(vg),valorsG);
		char[] valorsH={'F', 'E'};
		checkVertexs(graf.adjacents(vh),valorsH);
	}


}
