package uoc.ei.tads.tests;
import  uoc.ei.tads.*;
import uoc.ei.tads.RecorregutBidireccional.IniciRecorregut;
/**
 * Test d'alguns mtodes de la classe.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class TestLlistaDoblementEncadenada  extends TestLlistaEncadenada {
   /**
    * Constructor amb un parmetre. Delega en la superclasse.
    * @param name  nom del test
    */
   public TestLlistaDoblementEncadenada(String name) { super(name); }


   protected Contenidor<Integer> crearContenidor() {
	   return new LlistaDoblementEncadenada<Integer>();
   }


   public void testAfegirAlFinal() {
	   super.testAfegirAlFinal();
   }


   public void testAfegirAlPrincipi() {
	   super.testAfegirAlPrincipi();
   }


   public void testAfegirDespresDe() {
	   super.testAfegirDespresDe();
   }
   
   
   public void testEsborrarSeguent() {
	   super.testEsborrarSeguent();
   }
   

   public void testPosicions() {
	   super.testPosicions();
   }

   
   public void testEsborrar() {
	   super.testEsborrar();
   }
   
   
   public void testEsborrarPrimer() {
	   super.testEsborrarPrimer();
   }

   
   /** Comprova els RecorregutsBidireccionals comenant desde el principi
    * Crea una llista amb afegirAlPrincipi i afegirDespresDe,
    * Operacions comprovades: afegirAlPrincipi, afegirDespresDe, 
    * 	  posicions(int).
    */
   
   public void testRecorregutBidireccionalDesdePrincipi() {
      LlistaDoblementEncadenada<Integer> llista = (LlistaDoblementEncadenada<Integer>)crearContenidor();
      assertTrue(llista.estaBuit());
      RecorregutBidireccional<Integer> recorregut=llista.posicions(IniciRecorregut.PRINCIPI);
      assertFalse(recorregut.hiHaSeguent());
      assertFalse(recorregut.hiHaAnterior());
      Posicio<Integer> p1=llista.afegirAlPrincipi(1);
      recorregut=llista.posicions(IniciRecorregut.PRINCIPI);
      int elem=recorregut.seguent().getElem();
      assertEquals(1,elem);
      Posicio<Integer> p4=llista.afegirDespresDe(p1,4);
      Posicio<Integer> p2=llista.afegirDespresDe(p1,2);
      llista.afegirDespresDe(p2,3);
      llista.afegirDespresDe(p4,5);
      estaOrdenat(llista.elements());
      recorregut=llista.posicions(IniciRecorregut.PRINCIPI);
      for(int i=1;i<=5 && recorregut.hiHaSeguent();i++) {
    	  elem=recorregut.seguent().getElem();
    	  assertEquals(i,elem);
    	  assertTrue(recorregut.hiHaAnterior());
      }
      try {
    	  recorregut.seguent();
    	  fail("final recorregut: s'havia de llenar ExcepciPosicioInvlida");
      } catch (ExcepcioPosicioInvalida e) {}
      for(int i=4;i>=1 && recorregut.hiHaAnterior();i--) {
    	  elem=recorregut.anterior().getElem();
    	  assertEquals(i,elem);
    	  assertTrue(recorregut.hiHaSeguent());
      }
      try {
    	  recorregut.anterior();
    	  fail("final recorregut: s'havia de llenar ExcepciPosicioInvlida");
      } catch (ExcepcioPosicioInvalida e) {}
      for(int i=2;i<=5 && recorregut.hiHaSeguent();i++) {
    	  elem=recorregut.seguent().getElem();
    	  assertEquals(i,elem);
    	  assertTrue(recorregut.hiHaAnterior());
      }
   }
   
   
   /** Comprova els RecorregutsBidireccionals comenant desde el final
    * Crea una llista amb afegirAlPrincipi i afegirDespresDe,
    * Operacions comprovades: afegirAlPrincipi, afegirDespresDe, 
    * 	  posicions(int).
    */
   
   public void testRecorregutBidireccionalDesdeFinal() {
      LlistaDoblementEncadenada<Integer> llista = (LlistaDoblementEncadenada<Integer>)crearContenidor();
      assertTrue(llista.estaBuit());
      RecorregutBidireccional<Integer> recorregut=llista.posicions(IniciRecorregut.FINAL);
      assertFalse(recorregut.hiHaSeguent());
      assertFalse(recorregut.hiHaAnterior());
      Posicio<Integer> p1=llista.afegirAlPrincipi(1);
      recorregut=llista.posicions(IniciRecorregut.FINAL);
      int elem=recorregut.anterior().getElem();
      assertEquals(1,elem);
      Posicio<Integer> p4=llista.afegirDespresDe(p1,4);
      Posicio<Integer> p2=llista.afegirDespresDe(p1,2);
      llista.afegirDespresDe(p2,3);
      llista.afegirDespresDe(p4,5);
      estaOrdenat(llista.elements());
      recorregut=llista.posicions(IniciRecorregut.FINAL);
      for(int i=5;i>=1 && recorregut.hiHaAnterior();i--) {
    	  elem=recorregut.anterior().getElem();
    	  assertEquals(i,elem);
    	  assertTrue(recorregut.hiHaSeguent());
      }
      try {
    	  recorregut.anterior();
    	  fail("final recorregut: s'havia de llenar ExcepciPosicioInvlida");
      } catch (ExcepcioPosicioInvalida e) {}
      for(int i=2;i<=5 && recorregut.hiHaSeguent();i++) {
    	  elem=recorregut.seguent().getElem();
    	  assertEquals(i,elem);
    	  assertTrue(recorregut.hiHaAnterior());
      }
      try {
    	  recorregut.seguent();
    	  fail("final recorregut: s'havia de llenar ExcepciPosicioInvlida");
      } catch (ExcepcioPosicioInvalida e) {}
      for(int i=4;i>=1 && recorregut.hiHaAnterior();i--) {
    	  elem=recorregut.anterior().getElem();
    	  assertEquals(i,elem);
    	  assertTrue(recorregut.hiHaSeguent());
      }
   }
   
   
   /** Permet fer el test. */
   public static void main(String[] args) 
   {
      junit.textui.TestRunner.run(TestLlistaDoblementEncadenada.class);
   }
}
