package uoc.ei.tads.tests;
import  uoc.ei.tads.*;

/**
 * Test d'alguns mtodes de la classe.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */
public class TestLlistaEncadenada  extends TestContenidor
{
   /**
    * Constructor amb un parmetre. Delega en la superclasse.
    * @param name  nom del test
    */
   public TestLlistaEncadenada(String name) { super(name); }
   
   
   protected Contenidor<Integer> crearContenidor() {
	   return new LlistaEncadenada<Integer>();
   }


   /** Crea una llista amb afegirAlFinal, i n'elimina els elements amb esborrarPrimer.
    * Operacions comprovades: afegirAlFinal, esborrarPrimer, estaBuit.
    */
   
   public void testAfegirAlFinal()
   {
      Llista<Integer> llista = (Llista<Integer>)crearContenidor();
      assertTrue(llista.estaBuit());
      int elem=llista.afegirAlFinal(1).getElem();
      assertEquals(1,elem);
      elem=llista.afegirAlFinal(2).getElem();
      assertEquals(2,elem);
      elem=llista.afegirAlFinal(3).getElem();
      assertEquals(3,elem);
      elem=llista.afegirAlFinal(4).getElem();
      assertEquals(4,elem);
      assertEquals(4,llista.nombreElems());
      elem=llista.esborrarPrimer();
      assertEquals(1,elem);
      elem=llista.esborrarPrimer();
      assertEquals(2,elem);
      elem=llista.esborrarPrimer();
      assertEquals(3,elem);
      assertFalse(llista.estaBuit());
      elem=llista.esborrarPrimer();
      assertEquals(4,elem);
      try {
    	  llista.esborrarPrimer();
          fail("Expected ExcepcioContenidorBuit");
      } catch (ExcepcioContenidorBuit ex) {}
      assertTrue(llista.estaBuit());
   }


   /** Crea una llista amb afegirAlPrincipi, i n'elimina els elements amb esborrarPrimer.
    * Operacions comprovades: afegirAlPrincipi, esborrarPrimer, estaBuit.
    */
   
   public void testAfegirAlPrincipi()
   {
      Llista<Integer> llista = (Llista<Integer>)crearContenidor();
      assertTrue(llista.estaBuit());
      int elem=llista.afegirAlPrincipi(1).getElem();
      assertEquals(1,elem);
      elem=llista.afegirAlPrincipi(2).getElem();
      assertEquals(2,elem);
      elem=llista.afegirAlPrincipi(3).getElem();
      assertEquals(3,elem);
      elem=llista.afegirAlPrincipi(4).getElem();
      assertEquals(4,elem);
      assertEquals(4,llista.nombreElems());
      elem=llista.esborrarPrimer();
      assertEquals(4,elem);
      elem=llista.esborrarPrimer();
      assertEquals(3,elem);
      elem=llista.esborrarPrimer();
      assertEquals(2,elem);
      assertFalse(llista.estaBuit());
      elem=llista.esborrarPrimer();
      assertEquals(1,elem);
      try {
    	  llista.esborrarPrimer();
          fail("Expected ExcepcioContenidorBuit");
      } catch (ExcepcioContenidorBuit ex) {}
      assertTrue(llista.estaBuit());
   }


   /** Crea una llista amb afegirAlPrincipi i afegirDespresDe,
    * Operacions comprovades: afegirAlPrincipi, afegirDespresDe, reemplacar, intercanviar,
    * 	  esborrarSeguent, elements, estaBuit, nombreElements.
    */
   
   public void testAfegirDespresDe()
   {
      Llista<Integer> llista = (Llista<Integer>)crearContenidor();
      assertTrue(llista.estaBuit());
      Posicio<Integer> p1=llista.afegirAlPrincipi(1);
      Posicio<Integer> p4=llista.afegirDespresDe(p1,4);
      Posicio<Integer> p2=llista.afegirDespresDe(p1,2);
      Posicio<Integer> p3=llista.afegirDespresDe(p2,3);
      Posicio<Integer> p5=llista.afegirDespresDe(p4,5);
      estaOrdenat(llista.elements());
      int elem=p1.getElem();
      assertEquals(1,elem);
      elem=p2.getElem();
      assertEquals(2,elem);
      elem=p3.getElem();
      assertEquals(3,elem);
      elem=p4.getElem();
      assertEquals(4,elem);
      elem=p5.getElem();
      assertEquals(5,elem);
      llista.intercanviar(p2,p4);
      elem=p2.getElem();
      assertEquals(4,elem);
      elem=p4.getElem();
      assertEquals(2,elem);
      llista.reemplacar(p2,2);
      elem=p2.getElem();
      assertEquals(2,elem);
      assertEquals(5,llista.nombreElems());
      llista.esborrarSeguent(p3);
      assertEquals(4,llista.nombreElems());
      estaOrdenat(llista.elements());
   }
   
   
   /** Crea una llista amb afegirAlPrincipi i afegirDespresDe,
    * Operacions comprovades: afegirAlPrincipi, afegirDespresDe, 
    * 	  esborrarSeguent, elements, estaBuit, nombreElements.
    */
   
   public void testEsborrarSeguent()
   {
      Llista<Integer> llista = (Llista<Integer>)crearContenidor();
      assertTrue(llista.estaBuit());
      Posicio<Integer> p1=llista.afegirAlPrincipi(1);
      Posicio<Integer> p4=llista.afegirDespresDe(p1,4);
      Posicio<Integer> p2=llista.afegirDespresDe(p1,2);
      Posicio<Integer> p3=llista.afegirDespresDe(p2,3);
      Posicio<Integer> p5=llista.afegirDespresDe(p4,5);
      estaOrdenat(llista.elements());
      try {
    	  llista.esborrarSeguent(p5);
    	  fail("S'hauria d'haver llenat ExcepcioPosicioInvalida");
      } catch (ExcepcioPosicioInvalida e) {}
      int elem=llista.esborrarSeguent(p1);
      assertEquals(2,elem);
      elem=llista.esborrarSeguent(p3);
      assertEquals(4,elem);
      elem=llista.esborrarSeguent(p3);
      assertEquals(5,elem);
      assertEquals(2,llista.nombreElems());
      Iterador<Integer> elems=llista.elements();
      elem=elems.seguent();
      assertEquals(1,elem);
      elem=elems.seguent();
      assertEquals(3,elem);
      assertFalse(elems.hiHaSeguent());
   }
   
   
   /** Crea una llista amb afegirAlPrincipi i afegirDespresDe,
    * Operacions comprovades: afegirAlPrincipi, afegirDespresDe, 
    * 	  esborrar, elements, estaBuit, nombreElements.
    */
   
   public void testEsborrar()
   {
      Llista<Integer> llista = (Llista<Integer>)crearContenidor();
      assertTrue(llista.estaBuit());
      Posicio<Integer> p1=llista.afegirAlPrincipi(1);
      Posicio<Integer> p4=llista.afegirDespresDe(p1,4);
      Posicio<Integer> p2=llista.afegirDespresDe(p1,2);
      Posicio<Integer> p3=llista.afegirDespresDe(p2,3);
      Posicio<Integer> p5=llista.afegirDespresDe(p4,5);
      estaOrdenat(llista.elements());
      try {
    	  llista.esborrarSeguent(p5);
    	  fail("S'hauria d'haver llenat ExcepcioPosicioInvalida");
      } catch (ExcepcioPosicioInvalida e) {}
      int elem=llista.esborrar(p1);
      assertEquals(1,elem);
      elem=llista.esborrar(p3);
      assertEquals(3,elem);
      elem=llista.esborrar(p5);
      assertEquals(5,elem);
      assertEquals(2,llista.nombreElems());
      Iterador<Integer> elems=llista.elements();
      elem=elems.seguent();
      assertEquals(2,elem);
      elem=elems.seguent();
      assertEquals(4,elem);
      assertFalse(elems.hiHaSeguent());
      elem=llista.esborrar(p2);
      assertEquals(2,elem);
      elem=llista.esborrar(p4);
      assertEquals(4,elem);
      assertTrue(llista.estaBuit());
   }
   
   
   /** Prova posicions i elements amb 0, 1 i diversos elements.
    * Per a aquest darrer cas, crea una llista de 5 elements
    * amb afegirAlPrincipi i afegirDespresDe.
    * Operacions comprovades: afegirAlPrincipi, afegirDespresDe, 
    * 	  esborrarSeguent, elements, estaBuit, nombreElements.
    */
   
   public void testPosicions()
   {
      Llista<Integer> llista = (Llista<Integer>)crearContenidor();
      Iterador<Integer> iterador=llista.elements();
      Recorregut<Integer> recorregut=llista.posicions();
      assertTrue(llista.estaBuit());
      assertFalse(iterador.hiHaSeguent());
      assertFalse(recorregut.hiHaSeguent());
      Posicio<Integer> p1=llista.afegirAlPrincipi(1);
      iterador=llista.elements();
      recorregut=llista.posicions();
      int elemR=recorregut.seguent().getElem();
      int elemI=iterador.seguent();
      assertEquals(elemI,elemR);
      assertFalse(iterador.hiHaSeguent());
      assertFalse(recorregut.hiHaSeguent());
      Posicio<Integer> p4=llista.afegirDespresDe(p1,4);
      Posicio<Integer> p2=llista.afegirDespresDe(p1,2);
      llista.afegirDespresDe(p2,3);
      llista.afegirDespresDe(p4,5);
      estaOrdenat(llista.elements());
      iterador=llista.elements();
      recorregut=llista.posicions();
      while (recorregut.hiHaSeguent()) {
          elemR=recorregut.seguent().getElem();
          elemI=iterador.seguent();
          assertEquals(elemI,elemR);
      }
      assertFalse(iterador.hiHaSeguent());
      assertFalse(recorregut.hiHaSeguent());
   }
   
   
   /** Crea una llista amb afegirAlPrincipi i afegirDespresDe,
* Operacions comprovades: afegirAlPrincipi, afegirDespresDe, 
* 	  esborrar, elements, estaBuit, nombreElements.
*/
public void testEsborrarPrimer() {
	  Llista<Integer> llista = (Llista<Integer>)crearContenidor();
      assertTrue(llista.estaBuit());
      try {
    	  llista.esborrarPrimer();
    	  fail("S'hauria d'haver llenat ExcepcioContenidorBuit");
      } catch (ExcepcioContenidorBuit e) {}
      Posicio<Integer> p1=llista.afegirAlPrincipi(1);
      int elem=llista.esborrarPrimer();
      assertEquals(1,elem);
      assertTrue(llista.estaBuit());
      p1=llista.afegirAlPrincipi(1);
      Posicio<Integer> p4=llista.afegirDespresDe(p1,4);
      Posicio<Integer> p2=llista.afegirDespresDe(p1,2);
      llista.afegirDespresDe(p2,3);
      llista.afegirDespresDe(p4,5);
      estaOrdenat(llista.elements());
      for(int i=1;i<6 && !llista.estaBuit();i++) {
    	  elem=llista.esborrarPrimer();
    	  assertEquals(i,elem);
      }
      assertTrue(llista.estaBuit());
   }


/** Executa els tests fent servir una user interface textual. */
   public static void main(String[] args) 
   {
      junit.textui.TestRunner.run(TestLlistaEncadenada.class);
   }
}
