package uoc.ei.tads.tests;

import uoc.ei.tads.Contenidor;
import uoc.ei.tads.ExcepcioContenidorBuit;
import uoc.ei.tads.ExcepcioContenidorPle;
import uoc.ei.tads.Pila;
import uoc.ei.tads.PilaVectorImpl;

/** Conjunt de test cases sobre la implementaci amb vector del TAD pila.
 * PilaVectorImpl.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informaci,
 *          Universitat Oberta de Catalunya (UOC)
 * @version  2.0.0
 * @since  1.5
 */

public class TestPilaVectorImpl extends TestContenidor {


	protected Contenidor<Integer> crearContenidor() {
		return new PilaVectorImpl<Integer>();
	}


	protected Contenidor<Integer> crearContenidor(int numElems) {
		return new PilaVectorImpl<Integer>(numElems);
	}


	/** Comprova el llenament de l'excepci corresponent quan la pila s
	 * buida i intentem desempilar.
	 * Es comproven les operacions: empilar, desempilar, estaBuit.
	 */

	public void testBuit() {
		Pila<Integer> pila=(Pila<Integer>)crearContenidor();
		pila.empilar(5);
		int elem=pila.desempilar();
		assertEquals(5,elem);
		assertTrue(pila.estaBuit());
		try {
			pila.desempilar();
            fail("Expected ExcepcioContenidorBuit");
	} catch (ExcepcioContenidorBuit e) {}
		pila.empilar(8);
		elem=pila.desempilar();
		assertEquals(8,elem);
		assertTrue(pila.estaBuit());
	}


	/** Comprova el llenament de l'excepci corresponent quan la pila s
	 * plena i intentem empilar.
	 * Es comproven les operacions: empilar, desempilar, estaPle.
	 */

	public void testPle() {
		PilaVectorImpl<Integer> pila=(PilaVectorImpl<Integer>)crearContenidor(MIDA_PLENA);
		pila.empilar(5);
		pila.empilar(7);
		pila.empilar(9);
		assertTrue(pila.estaPle());
		try {
			pila.empilar(12);
			fail("Expected ExcepcioContenidorPle");
		} catch (ExcepcioContenidorPle e) {}
		int elem=pila.desempilar();
		assertEquals(9,elem);
		elem=pila.desempilar();
		assertEquals(7,elem);
		elem=pila.desempilar();
		assertEquals(5,elem);
		assertTrue(pila.estaBuit());
	}


	/** Comprova que els elements s'empilen en una pila i es desempilen
	 * en l'ordre esperat.
	 * Es comproven les operacions: empilar, desempilar.
	 */

	public void testOrdre() {
		Pila<Integer> pila=(Pila<Integer>)crearContenidor();
		pila.empilar(5);
		pila.empilar(7);
		pila.empilar(9);
		int elem=pila.desempilar();
		assertEquals(9,elem);
		elem=pila.desempilar();
		assertEquals(7,elem);
		elem=pila.desempilar();
		assertEquals(5,elem);
	}


	/** Aquest test realitza moltes operacions d'empilar i desempilar
	 * sobre una mateixa pila. Omple i buida la pila un nombre elevat
	 * de cops, comprovant cada cop que l'ordre en el que s'obtenen els
	 * elements s el correcte.
	 * Es comproven les operacions: empilar, desempilar, estaBuit, estaPle, cim.
	 */

	public void testMoltesOperacions() {
		PilaVectorImpl<Integer> pila=(PilaVectorImpl<Integer>)crearContenidor(MIDA_MOLTES_OPS);
		for (int i=0;i<REPETICIONS_MOLTES_OPS;i++) {
			for(int j=0;j<MIDA_MOLTES_OPS;j++) {
				pila.empilar(j);
				int elem=pila.cim();
				assertEquals(j,elem);
			}
			assertTrue(pila.estaPle());
			for(int j=0;j<MIDA_MOLTES_OPS;j++) {
				int elem=pila.desempilar();
				assertEquals(MIDA_MOLTES_OPS-j-1,elem);
			}
			assertTrue(pila.estaBuit());
		}
	}


}
